from collections.abc import Mapping
from typing import Any, TypeVar

import google.protobuf.message
import proto

from google.ads.googleads.v13.common.types.criteria import KeywordInfo
from google.ads.googleads.v13.enums.types.ad_destination_type import (
    AdDestinationTypeEnum,
)
from google.ads.googleads.v13.enums.types.ad_network_type import AdNetworkTypeEnum
from google.ads.googleads.v13.enums.types.budget_campaign_association_status import (
    BudgetCampaignAssociationStatusEnum,
)
from google.ads.googleads.v13.enums.types.click_type import ClickTypeEnum
from google.ads.googleads.v13.enums.types.conversion_action_category import (
    ConversionActionCategoryEnum,
)
from google.ads.googleads.v13.enums.types.conversion_attribution_event_type import (
    ConversionAttributionEventTypeEnum,
)
from google.ads.googleads.v13.enums.types.conversion_lag_bucket import (
    ConversionLagBucketEnum,
)
from google.ads.googleads.v13.enums.types.conversion_or_adjustment_lag_bucket import (
    ConversionOrAdjustmentLagBucketEnum,
)
from google.ads.googleads.v13.enums.types.conversion_value_rule_primary_dimension import (
    ConversionValueRulePrimaryDimensionEnum,
)
from google.ads.googleads.v13.enums.types.day_of_week import DayOfWeekEnum
from google.ads.googleads.v13.enums.types.device import DeviceEnum
from google.ads.googleads.v13.enums.types.external_conversion_source import (
    ExternalConversionSourceEnum,
)
from google.ads.googleads.v13.enums.types.hotel_date_selection_type import (
    HotelDateSelectionTypeEnum,
)
from google.ads.googleads.v13.enums.types.hotel_price_bucket import HotelPriceBucketEnum
from google.ads.googleads.v13.enums.types.hotel_rate_type import HotelRateTypeEnum
from google.ads.googleads.v13.enums.types.month_of_year import MonthOfYearEnum
from google.ads.googleads.v13.enums.types.placeholder_type import PlaceholderTypeEnum
from google.ads.googleads.v13.enums.types.product_channel import ProductChannelEnum
from google.ads.googleads.v13.enums.types.product_channel_exclusivity import (
    ProductChannelExclusivityEnum,
)
from google.ads.googleads.v13.enums.types.product_condition import ProductConditionEnum
from google.ads.googleads.v13.enums.types.recommendation_type import (
    RecommendationTypeEnum,
)
from google.ads.googleads.v13.enums.types.search_engine_results_page_type import (
    SearchEngineResultsPageTypeEnum,
)
from google.ads.googleads.v13.enums.types.search_term_match_type import (
    SearchTermMatchTypeEnum,
)
from google.ads.googleads.v13.enums.types.sk_ad_network_ad_event_type import (
    SkAdNetworkAdEventTypeEnum,
)
from google.ads.googleads.v13.enums.types.sk_ad_network_attribution_credit import (
    SkAdNetworkAttributionCreditEnum,
)
from google.ads.googleads.v13.enums.types.sk_ad_network_user_type import (
    SkAdNetworkUserTypeEnum,
)
from google.ads.googleads.v13.enums.types.slot import SlotEnum

_M = TypeVar("_M")

class AssetInteractionTarget(proto.Message):
    asset: str
    interaction_on_this_asset: bool
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        asset: str = ...,
        interaction_on_this_asset: bool = ...
    ) -> None: ...

class BudgetCampaignAssociationStatus(proto.Message):
    campaign: str
    status: BudgetCampaignAssociationStatusEnum.BudgetCampaignAssociationStatus
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        campaign: str = ...,
        status: BudgetCampaignAssociationStatusEnum.BudgetCampaignAssociationStatus = ...
    ) -> None: ...

class Keyword(proto.Message):
    ad_group_criterion: str
    info: KeywordInfo
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        ad_group_criterion: str = ...,
        info: KeywordInfo = ...
    ) -> None: ...

class Segments(proto.Message):
    activity_account_id: int
    activity_rating: int
    external_activity_id: str
    ad_destination_type: AdDestinationTypeEnum.AdDestinationType
    ad_network_type: AdNetworkTypeEnum.AdNetworkType
    auction_insight_domain: str
    budget_campaign_association_status: BudgetCampaignAssociationStatus
    click_type: ClickTypeEnum.ClickType
    conversion_action: str
    conversion_action_category: ConversionActionCategoryEnum.ConversionActionCategory
    conversion_action_name: str
    conversion_adjustment: bool
    conversion_attribution_event_type: ConversionAttributionEventTypeEnum.ConversionAttributionEventType
    conversion_lag_bucket: ConversionLagBucketEnum.ConversionLagBucket
    conversion_or_adjustment_lag_bucket: ConversionOrAdjustmentLagBucketEnum.ConversionOrAdjustmentLagBucket
    date: str
    day_of_week: DayOfWeekEnum.DayOfWeek
    device: DeviceEnum.Device
    external_conversion_source: ExternalConversionSourceEnum.ExternalConversionSource
    geo_target_airport: str
    geo_target_canton: str
    geo_target_city: str
    geo_target_country: str
    geo_target_county: str
    geo_target_district: str
    geo_target_metro: str
    geo_target_most_specific_location: str
    geo_target_postal_code: str
    geo_target_province: str
    geo_target_region: str
    geo_target_state: str
    hotel_booking_window_days: int
    hotel_center_id: int
    hotel_check_in_date: str
    hotel_check_in_day_of_week: DayOfWeekEnum.DayOfWeek
    hotel_city: str
    hotel_class: int
    hotel_country: str
    hotel_date_selection_type: HotelDateSelectionTypeEnum.HotelDateSelectionType
    hotel_length_of_stay: int
    hotel_rate_rule_id: str
    hotel_rate_type: HotelRateTypeEnum.HotelRateType
    hotel_price_bucket: HotelPriceBucketEnum.HotelPriceBucket
    hotel_state: str
    hour: int
    interaction_on_this_extension: bool
    keyword: Keyword
    month: str
    month_of_year: MonthOfYearEnum.MonthOfYear
    partner_hotel_id: str
    placeholder_type: PlaceholderTypeEnum.PlaceholderType
    product_aggregator_id: int
    product_bidding_category_level1: str
    product_bidding_category_level2: str
    product_bidding_category_level3: str
    product_bidding_category_level4: str
    product_bidding_category_level5: str
    product_brand: str
    product_channel: ProductChannelEnum.ProductChannel
    product_channel_exclusivity: ProductChannelExclusivityEnum.ProductChannelExclusivity
    product_condition: ProductConditionEnum.ProductCondition
    product_country: str
    product_custom_attribute0: str
    product_custom_attribute1: str
    product_custom_attribute2: str
    product_custom_attribute3: str
    product_custom_attribute4: str
    product_feed_label: str
    product_item_id: str
    product_language: str
    product_merchant_id: int
    product_store_id: str
    product_title: str
    product_type_l1: str
    product_type_l2: str
    product_type_l3: str
    product_type_l4: str
    product_type_l5: str
    quarter: str
    recommendation_type: RecommendationTypeEnum.RecommendationType
    search_engine_results_page_type: SearchEngineResultsPageTypeEnum.SearchEngineResultsPageType
    search_term_match_type: SearchTermMatchTypeEnum.SearchTermMatchType
    slot: SlotEnum.Slot
    conversion_value_rule_primary_dimension: ConversionValueRulePrimaryDimensionEnum.ConversionValueRulePrimaryDimension
    webpage: str
    week: str
    year: int
    sk_ad_network_conversion_value: int
    sk_ad_network_user_type: SkAdNetworkUserTypeEnum.SkAdNetworkUserType
    sk_ad_network_ad_event_type: SkAdNetworkAdEventTypeEnum.SkAdNetworkAdEventType
    sk_ad_network_source_app: SkAdNetworkSourceApp
    sk_ad_network_attribution_credit: SkAdNetworkAttributionCreditEnum.SkAdNetworkAttributionCredit
    asset_interaction_target: AssetInteractionTarget
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        activity_account_id: int = ...,
        activity_rating: int = ...,
        external_activity_id: str = ...,
        ad_destination_type: AdDestinationTypeEnum.AdDestinationType = ...,
        ad_network_type: AdNetworkTypeEnum.AdNetworkType = ...,
        auction_insight_domain: str = ...,
        budget_campaign_association_status: BudgetCampaignAssociationStatus = ...,
        click_type: ClickTypeEnum.ClickType = ...,
        conversion_action: str = ...,
        conversion_action_category: ConversionActionCategoryEnum.ConversionActionCategory = ...,
        conversion_action_name: str = ...,
        conversion_adjustment: bool = ...,
        conversion_attribution_event_type: ConversionAttributionEventTypeEnum.ConversionAttributionEventType = ...,
        conversion_lag_bucket: ConversionLagBucketEnum.ConversionLagBucket = ...,
        conversion_or_adjustment_lag_bucket: ConversionOrAdjustmentLagBucketEnum.ConversionOrAdjustmentLagBucket = ...,
        date: str = ...,
        day_of_week: DayOfWeekEnum.DayOfWeek = ...,
        device: DeviceEnum.Device = ...,
        external_conversion_source: ExternalConversionSourceEnum.ExternalConversionSource = ...,
        geo_target_airport: str = ...,
        geo_target_canton: str = ...,
        geo_target_city: str = ...,
        geo_target_country: str = ...,
        geo_target_county: str = ...,
        geo_target_district: str = ...,
        geo_target_metro: str = ...,
        geo_target_most_specific_location: str = ...,
        geo_target_postal_code: str = ...,
        geo_target_province: str = ...,
        geo_target_region: str = ...,
        geo_target_state: str = ...,
        hotel_booking_window_days: int = ...,
        hotel_center_id: int = ...,
        hotel_check_in_date: str = ...,
        hotel_check_in_day_of_week: DayOfWeekEnum.DayOfWeek = ...,
        hotel_city: str = ...,
        hotel_class: int = ...,
        hotel_country: str = ...,
        hotel_date_selection_type: HotelDateSelectionTypeEnum.HotelDateSelectionType = ...,
        hotel_length_of_stay: int = ...,
        hotel_rate_rule_id: str = ...,
        hotel_rate_type: HotelRateTypeEnum.HotelRateType = ...,
        hotel_price_bucket: HotelPriceBucketEnum.HotelPriceBucket = ...,
        hotel_state: str = ...,
        hour: int = ...,
        interaction_on_this_extension: bool = ...,
        keyword: Keyword = ...,
        month: str = ...,
        month_of_year: MonthOfYearEnum.MonthOfYear = ...,
        partner_hotel_id: str = ...,
        placeholder_type: PlaceholderTypeEnum.PlaceholderType = ...,
        product_aggregator_id: int = ...,
        product_bidding_category_level1: str = ...,
        product_bidding_category_level2: str = ...,
        product_bidding_category_level3: str = ...,
        product_bidding_category_level4: str = ...,
        product_bidding_category_level5: str = ...,
        product_brand: str = ...,
        product_channel: ProductChannelEnum.ProductChannel = ...,
        product_channel_exclusivity: ProductChannelExclusivityEnum.ProductChannelExclusivity = ...,
        product_condition: ProductConditionEnum.ProductCondition = ...,
        product_country: str = ...,
        product_custom_attribute0: str = ...,
        product_custom_attribute1: str = ...,
        product_custom_attribute2: str = ...,
        product_custom_attribute3: str = ...,
        product_custom_attribute4: str = ...,
        product_feed_label: str = ...,
        product_item_id: str = ...,
        product_language: str = ...,
        product_merchant_id: int = ...,
        product_store_id: str = ...,
        product_title: str = ...,
        product_type_l1: str = ...,
        product_type_l2: str = ...,
        product_type_l3: str = ...,
        product_type_l4: str = ...,
        product_type_l5: str = ...,
        quarter: str = ...,
        recommendation_type: RecommendationTypeEnum.RecommendationType = ...,
        search_engine_results_page_type: SearchEngineResultsPageTypeEnum.SearchEngineResultsPageType = ...,
        search_term_match_type: SearchTermMatchTypeEnum.SearchTermMatchType = ...,
        slot: SlotEnum.Slot = ...,
        conversion_value_rule_primary_dimension: ConversionValueRulePrimaryDimensionEnum.ConversionValueRulePrimaryDimension = ...,
        webpage: str = ...,
        week: str = ...,
        year: int = ...,
        sk_ad_network_conversion_value: int = ...,
        sk_ad_network_user_type: SkAdNetworkUserTypeEnum.SkAdNetworkUserType = ...,
        sk_ad_network_ad_event_type: SkAdNetworkAdEventTypeEnum.SkAdNetworkAdEventType = ...,
        sk_ad_network_source_app: SkAdNetworkSourceApp = ...,
        sk_ad_network_attribution_credit: SkAdNetworkAttributionCreditEnum.SkAdNetworkAttributionCredit = ...,
        asset_interaction_target: AssetInteractionTarget = ...
    ) -> None: ...

class SkAdNetworkSourceApp(proto.Message):
    sk_ad_network_source_app_id: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        sk_ad_network_source_app_id: str = ...
    ) -> None: ...



from collections.abc import Mapping, MutableSequence
from typing import Any, TypeVar

import google.protobuf.message
import proto

from google.ads.googleads.v14.common.types.criteria import KeywordInfo
from google.ads.googleads.v14.common.types.dates import DateRange
from google.ads.googleads.v14.common.types.keyword_plan_common import (
    HistoricalMetricsOptions,
    KeywordAnnotations,
    KeywordPlanAggregateMetricResults,
    KeywordPlanAggregateMetrics,
    KeywordPlanHistoricalMetrics,
)
from google.ads.googleads.v14.enums.types.keyword_match_type import KeywordMatchTypeEnum
from google.ads.googleads.v14.enums.types.keyword_plan_keyword_annotation import (
    KeywordPlanKeywordAnnotationEnum,
)
from google.ads.googleads.v14.enums.types.keyword_plan_network import (
    KeywordPlanNetworkEnum,
)

_M = TypeVar("_M")

class AdGroupKeywordSuggestion(proto.Message):
    keyword_text: str
    suggested_keyword_text: str
    suggested_match_type: KeywordMatchTypeEnum.KeywordMatchType
    suggested_ad_group: str
    suggested_campaign: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        keyword_text: str = ...,
        suggested_keyword_text: str = ...,
        suggested_match_type: KeywordMatchTypeEnum.KeywordMatchType = ...,
        suggested_ad_group: str = ...,
        suggested_campaign: str = ...
    ) -> None: ...

class BiddableKeyword(proto.Message):
    keyword: KeywordInfo
    max_cpc_bid_micros: int
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        keyword: KeywordInfo = ...,
        max_cpc_bid_micros: int = ...
    ) -> None: ...

class CampaignToForecast(proto.Message):
    class CampaignBiddingStrategy(proto.Message):
        manual_cpc_bidding_strategy: ManualCpcBiddingStrategy
        maximize_clicks_bidding_strategy: MaximizeClicksBiddingStrategy
        maximize_conversions_bidding_strategy: MaximizeConversionsBiddingStrategy
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            manual_cpc_bidding_strategy: ManualCpcBiddingStrategy = ...,
            maximize_clicks_bidding_strategy: MaximizeClicksBiddingStrategy = ...,
            maximize_conversions_bidding_strategy: MaximizeConversionsBiddingStrategy = ...
        ) -> None: ...
    language_constants: MutableSequence[str]
    geo_modifiers: MutableSequence[CriterionBidModifier]
    keyword_plan_network: KeywordPlanNetworkEnum.KeywordPlanNetwork
    negative_keywords: MutableSequence[KeywordInfo]
    bidding_strategy: CampaignToForecast.CampaignBiddingStrategy
    conversion_rate: float
    ad_groups: MutableSequence[ForecastAdGroup]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        language_constants: MutableSequence[str] = ...,
        geo_modifiers: MutableSequence[CriterionBidModifier] = ...,
        keyword_plan_network: KeywordPlanNetworkEnum.KeywordPlanNetwork = ...,
        negative_keywords: MutableSequence[KeywordInfo] = ...,
        bidding_strategy: CampaignToForecast.CampaignBiddingStrategy = ...,
        conversion_rate: float = ...,
        ad_groups: MutableSequence[ForecastAdGroup] = ...
    ) -> None: ...

class CriterionBidModifier(proto.Message):
    geo_target_constant: str
    bid_modifier: float
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        geo_target_constant: str = ...,
        bid_modifier: float = ...
    ) -> None: ...

class ForecastAdGroup(proto.Message):
    max_cpc_bid_micros: int
    biddable_keywords: MutableSequence[BiddableKeyword]
    negative_keywords: MutableSequence[KeywordInfo]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        max_cpc_bid_micros: int = ...,
        biddable_keywords: MutableSequence[BiddableKeyword] = ...,
        negative_keywords: MutableSequence[KeywordInfo] = ...
    ) -> None: ...

class GenerateAdGroupThemesRequest(proto.Message):
    customer_id: str
    keywords: MutableSequence[str]
    ad_groups: MutableSequence[str]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        customer_id: str = ...,
        keywords: MutableSequence[str] = ...,
        ad_groups: MutableSequence[str] = ...
    ) -> None: ...

class GenerateAdGroupThemesResponse(proto.Message):
    ad_group_keyword_suggestions: MutableSequence[AdGroupKeywordSuggestion]
    unusable_ad_groups: MutableSequence[UnusableAdGroup]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        ad_group_keyword_suggestions: MutableSequence[AdGroupKeywordSuggestion] = ...,
        unusable_ad_groups: MutableSequence[UnusableAdGroup] = ...
    ) -> None: ...

class GenerateKeywordForecastMetricsRequest(proto.Message):
    customer_id: str
    currency_code: str
    forecast_period: DateRange
    campaign: CampaignToForecast
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        customer_id: str = ...,
        currency_code: str = ...,
        forecast_period: DateRange = ...,
        campaign: CampaignToForecast = ...
    ) -> None: ...

class GenerateKeywordForecastMetricsResponse(proto.Message):
    campaign_forecast_metrics: KeywordForecastMetrics
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        campaign_forecast_metrics: KeywordForecastMetrics = ...
    ) -> None: ...

class GenerateKeywordHistoricalMetricsRequest(proto.Message):
    customer_id: str
    keywords: MutableSequence[str]
    language: str
    include_adult_keywords: bool
    geo_target_constants: MutableSequence[str]
    keyword_plan_network: KeywordPlanNetworkEnum.KeywordPlanNetwork
    aggregate_metrics: KeywordPlanAggregateMetrics
    historical_metrics_options: HistoricalMetricsOptions
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        customer_id: str = ...,
        keywords: MutableSequence[str] = ...,
        language: str = ...,
        include_adult_keywords: bool = ...,
        geo_target_constants: MutableSequence[str] = ...,
        keyword_plan_network: KeywordPlanNetworkEnum.KeywordPlanNetwork = ...,
        aggregate_metrics: KeywordPlanAggregateMetrics = ...,
        historical_metrics_options: HistoricalMetricsOptions = ...
    ) -> None: ...

class GenerateKeywordHistoricalMetricsResponse(proto.Message):
    results: MutableSequence[GenerateKeywordHistoricalMetricsResult]
    aggregate_metric_results: KeywordPlanAggregateMetricResults
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        results: MutableSequence[GenerateKeywordHistoricalMetricsResult] = ...,
        aggregate_metric_results: KeywordPlanAggregateMetricResults = ...
    ) -> None: ...

class GenerateKeywordHistoricalMetricsResult(proto.Message):
    text: str
    close_variants: MutableSequence[str]
    keyword_metrics: KeywordPlanHistoricalMetrics
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        text: str = ...,
        close_variants: MutableSequence[str] = ...,
        keyword_metrics: KeywordPlanHistoricalMetrics = ...
    ) -> None: ...

class GenerateKeywordIdeaResponse(proto.Message):
    results: MutableSequence[GenerateKeywordIdeaResult]
    aggregate_metric_results: KeywordPlanAggregateMetricResults
    next_page_token: str
    total_size: int
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        results: MutableSequence[GenerateKeywordIdeaResult] = ...,
        aggregate_metric_results: KeywordPlanAggregateMetricResults = ...,
        next_page_token: str = ...,
        total_size: int = ...
    ) -> None: ...

class GenerateKeywordIdeaResult(proto.Message):
    text: str
    keyword_idea_metrics: KeywordPlanHistoricalMetrics
    keyword_annotations: KeywordAnnotations
    close_variants: MutableSequence[str]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        text: str = ...,
        keyword_idea_metrics: KeywordPlanHistoricalMetrics = ...,
        keyword_annotations: KeywordAnnotations = ...,
        close_variants: MutableSequence[str] = ...
    ) -> None: ...

class GenerateKeywordIdeasRequest(proto.Message):
    customer_id: str
    language: str
    geo_target_constants: MutableSequence[str]
    include_adult_keywords: bool
    page_token: str
    page_size: int
    keyword_plan_network: KeywordPlanNetworkEnum.KeywordPlanNetwork
    keyword_annotation: MutableSequence[
        KeywordPlanKeywordAnnotationEnum.KeywordPlanKeywordAnnotation
    ]
    aggregate_metrics: KeywordPlanAggregateMetrics
    historical_metrics_options: HistoricalMetricsOptions
    keyword_and_url_seed: KeywordAndUrlSeed
    keyword_seed: KeywordSeed
    url_seed: UrlSeed
    site_seed: SiteSeed
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        customer_id: str = ...,
        language: str = ...,
        geo_target_constants: MutableSequence[str] = ...,
        include_adult_keywords: bool = ...,
        page_token: str = ...,
        page_size: int = ...,
        keyword_plan_network: KeywordPlanNetworkEnum.KeywordPlanNetwork = ...,
        keyword_annotation: MutableSequence[
            KeywordPlanKeywordAnnotationEnum.KeywordPlanKeywordAnnotation
        ] = ...,
        aggregate_metrics: KeywordPlanAggregateMetrics = ...,
        historical_metrics_options: HistoricalMetricsOptions = ...,
        keyword_and_url_seed: KeywordAndUrlSeed = ...,
        keyword_seed: KeywordSeed = ...,
        url_seed: UrlSeed = ...,
        site_seed: SiteSeed = ...
    ) -> None: ...

class KeywordAndUrlSeed(proto.Message):
    url: str
    keywords: MutableSequence[str]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        url: str = ...,
        keywords: MutableSequence[str] = ...
    ) -> None: ...

class KeywordForecastMetrics(proto.Message):
    impressions: float
    click_through_rate: float
    average_cpc_micros: int
    clicks: float
    cost_micros: int
    conversions: float
    conversion_rate: float
    average_cpa_micros: int
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        impressions: float = ...,
        click_through_rate: float = ...,
        average_cpc_micros: int = ...,
        clicks: float = ...,
        cost_micros: int = ...,
        conversions: float = ...,
        conversion_rate: float = ...,
        average_cpa_micros: int = ...
    ) -> None: ...

class KeywordSeed(proto.Message):
    keywords: MutableSequence[str]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        keywords: MutableSequence[str] = ...
    ) -> None: ...

class ManualCpcBiddingStrategy(proto.Message):
    daily_budget_micros: int
    max_cpc_bid_micros: int
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        daily_budget_micros: int = ...,
        max_cpc_bid_micros: int = ...
    ) -> None: ...

class MaximizeClicksBiddingStrategy(proto.Message):
    daily_target_spend_micros: int
    max_cpc_bid_ceiling_micros: int
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        daily_target_spend_micros: int = ...,
        max_cpc_bid_ceiling_micros: int = ...
    ) -> None: ...

class MaximizeConversionsBiddingStrategy(proto.Message):
    daily_target_spend_micros: int
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        daily_target_spend_micros: int = ...
    ) -> None: ...

class SiteSeed(proto.Message):
    site: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        site: str = ...
    ) -> None: ...

class UnusableAdGroup(proto.Message):
    ad_group: str
    campaign: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        ad_group: str = ...,
        campaign: str = ...
    ) -> None: ...

class UrlSeed(proto.Message):
    url: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        url: str = ...
    ) -> None: ...



from collections.abc import Mapping
from typing import Any, TypeVar

import google.protobuf.message
import proto

_M = TypeVar("_M")

class MonthOfYearEnum(proto.Message):
    class MonthOfYear(proto.Enum):
        UNSPECIFIED = 0
        UNKNOWN = 1
        JANUARY = 2
        FEBRUARY = 3
        MARCH = 4
        APRIL = 5
        MAY = 6
        JUNE = 7
        JULY = 8
        AUGUST = 9
        SEPTEMBER = 10
        OCTOBER = 11
        NOVEMBER = 12
        DECEMBER = 13
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
    ) -> None: ...



from typing import Any, Callable, Iterable, Iterator, Sequence, Tuple

from google.ads.googleads.v14.services.types import (
    keyword_plan_idea_service as keyword_plan_idea_service,
)

class GenerateKeywordIdeasPager:
    def __init__(
        self,
        method: Callable[..., keyword_plan_idea_service.GenerateKeywordIdeaResponse],
        request: keyword_plan_idea_service.GenerateKeywordIdeasRequest,
        response: keyword_plan_idea_service.GenerateKeywordIdeaResponse,
        metadata: Sequence[Tuple[str, str]] = ...,
    ) -> None: ...
    def __getattr__(self, name: str) -> Any: ...
    @property
    def pages(
        self,
    ) -> Iterable[keyword_plan_idea_service.GenerateKeywordIdeaResponse]: ...
    def __iter__(
        self,
    ) -> Iterator[keyword_plan_idea_service.GenerateKeywordIdeaResult]: ...



from collections.abc import Mapping, MutableSequence
from typing import Any, TypeVar

import google.protobuf.message
import proto
from google.protobuf.field_mask_pb2 import FieldMask

from google.ads.googleads.v13.resources.types.merchant_center_link import (
    MerchantCenterLink,
)

_M = TypeVar("_M")

class GetMerchantCenterLinkRequest(proto.Message):
    resource_name: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        resource_name: str = ...
    ) -> None: ...

class ListMerchantCenterLinksRequest(proto.Message):
    customer_id: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        customer_id: str = ...
    ) -> None: ...

class ListMerchantCenterLinksResponse(proto.Message):
    merchant_center_links: MutableSequence[MerchantCenterLink]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        merchant_center_links: MutableSequence[MerchantCenterLink] = ...
    ) -> None: ...

class MerchantCenterLinkOperation(proto.Message):
    update_mask: FieldMask
    update: MerchantCenterLink
    remove: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        update_mask: FieldMask = ...,
        update: MerchantCenterLink = ...,
        remove: str = ...
    ) -> None: ...

class MutateMerchantCenterLinkRequest(proto.Message):
    customer_id: str
    operation: MerchantCenterLinkOperation
    validate_only: bool
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        customer_id: str = ...,
        operation: MerchantCenterLinkOperation = ...,
        validate_only: bool = ...
    ) -> None: ...

class MutateMerchantCenterLinkResponse(proto.Message):
    result: MutateMerchantCenterLinkResult
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        result: MutateMerchantCenterLinkResult = ...
    ) -> None: ...

class MutateMerchantCenterLinkResult(proto.Message):
    resource_name: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        resource_name: str = ...
    ) -> None: ...



from typing import Any, Sequence, Union, overload

import proto
from google.protobuf.message import Message

def get_nested_attr(obj: Any, attr: str, *args: Any) -> Any: ...
def set_nested_message_field(
    message: Message, field_path: Union[str, Sequence[str]], value: Any
) -> None: ...
def convert_upper_case_to_snake_case(string: str) -> str: ...
def convert_snake_case_to_upper_case(string: str) -> str: ...
def convert_proto_plus_to_protobuf(
    message: Union[proto.Message, Message]
) -> Message: ...
def convert_protobuf_to_proto_plus(
    message: Union[Message, proto.Message]
) -> proto.Message: ...
def proto_copy_from(
    destination: Union[proto.Message, Message], origin: Union[proto.Message, Message]
) -> None: ...



from collections.abc import Mapping
from typing import Any, TypeVar

import google.protobuf.message
import proto

from google.ads.googleads.v15.enums.types.conversion_custom_variable_status import (
    ConversionCustomVariableStatusEnum,
)

_M = TypeVar("_M")

class ConversionCustomVariable(proto.Message):
    resource_name: str
    id: int
    name: str
    tag: str
    status: ConversionCustomVariableStatusEnum.ConversionCustomVariableStatus
    owner_customer: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        resource_name: str = ...,
        id: int = ...,
        name: str = ...,
        tag: str = ...,
        status: ConversionCustomVariableStatusEnum.ConversionCustomVariableStatus = ...,
        owner_customer: str = ...
    ) -> None: ...



from collections.abc import Mapping, MutableSequence
from typing import Any, TypeVar

import google.protobuf.message
import proto
from google.protobuf.duration_pb2 import Duration

from google.ads.googleads.v13.common.types.policy import (
    PolicyTopicEntry,
    PolicyViolationKey,
)
from google.ads.googleads.v13.common.types.value import Value
from google.ads.googleads.v13.enums.types.resource_limit_type import (
    ResourceLimitTypeEnum,
)
from google.ads.googleads.v13.errors.types.access_invitation_error import (
    AccessInvitationErrorEnum,
)
from google.ads.googleads.v13.errors.types.account_budget_proposal_error import (
    AccountBudgetProposalErrorEnum,
)
from google.ads.googleads.v13.errors.types.account_link_error import (
    AccountLinkErrorEnum,
)
from google.ads.googleads.v13.errors.types.ad_customizer_error import (
    AdCustomizerErrorEnum,
)
from google.ads.googleads.v13.errors.types.ad_error import AdErrorEnum
from google.ads.googleads.v13.errors.types.ad_group_ad_error import AdGroupAdErrorEnum
from google.ads.googleads.v13.errors.types.ad_group_bid_modifier_error import (
    AdGroupBidModifierErrorEnum,
)
from google.ads.googleads.v13.errors.types.ad_group_criterion_customizer_error import (
    AdGroupCriterionCustomizerErrorEnum,
)
from google.ads.googleads.v13.errors.types.ad_group_criterion_error import (
    AdGroupCriterionErrorEnum,
)
from google.ads.googleads.v13.errors.types.ad_group_customizer_error import (
    AdGroupCustomizerErrorEnum,
)
from google.ads.googleads.v13.errors.types.ad_group_error import AdGroupErrorEnum
from google.ads.googleads.v13.errors.types.ad_group_feed_error import (
    AdGroupFeedErrorEnum,
)
from google.ads.googleads.v13.errors.types.ad_parameter_error import (
    AdParameterErrorEnum,
)
from google.ads.googleads.v13.errors.types.ad_sharing_error import AdSharingErrorEnum
from google.ads.googleads.v13.errors.types.adx_error import AdxErrorEnum
from google.ads.googleads.v13.errors.types.asset_error import AssetErrorEnum
from google.ads.googleads.v13.errors.types.asset_group_asset_error import (
    AssetGroupAssetErrorEnum,
)
from google.ads.googleads.v13.errors.types.asset_group_error import AssetGroupErrorEnum
from google.ads.googleads.v13.errors.types.asset_group_listing_group_filter_error import (
    AssetGroupListingGroupFilterErrorEnum,
)
from google.ads.googleads.v13.errors.types.asset_link_error import AssetLinkErrorEnum
from google.ads.googleads.v13.errors.types.asset_set_asset_error import (
    AssetSetAssetErrorEnum,
)
from google.ads.googleads.v13.errors.types.asset_set_error import AssetSetErrorEnum
from google.ads.googleads.v13.errors.types.asset_set_link_error import (
    AssetSetLinkErrorEnum,
)
from google.ads.googleads.v13.errors.types.audience_error import AudienceErrorEnum
from google.ads.googleads.v13.errors.types.audience_insights_error import (
    AudienceInsightsErrorEnum,
)
from google.ads.googleads.v13.errors.types.authentication_error import (
    AuthenticationErrorEnum,
)
from google.ads.googleads.v13.errors.types.authorization_error import (
    AuthorizationErrorEnum,
)
from google.ads.googleads.v13.errors.types.batch_job_error import BatchJobErrorEnum
from google.ads.googleads.v13.errors.types.bidding_error import BiddingErrorEnum
from google.ads.googleads.v13.errors.types.bidding_strategy_error import (
    BiddingStrategyErrorEnum,
)
from google.ads.googleads.v13.errors.types.billing_setup_error import (
    BillingSetupErrorEnum,
)
from google.ads.googleads.v13.errors.types.campaign_budget_error import (
    CampaignBudgetErrorEnum,
)
from google.ads.googleads.v13.errors.types.campaign_conversion_goal_error import (
    CampaignConversionGoalErrorEnum,
)
from google.ads.googleads.v13.errors.types.campaign_criterion_error import (
    CampaignCriterionErrorEnum,
)
from google.ads.googleads.v13.errors.types.campaign_customizer_error import (
    CampaignCustomizerErrorEnum,
)
from google.ads.googleads.v13.errors.types.campaign_draft_error import (
    CampaignDraftErrorEnum,
)
from google.ads.googleads.v13.errors.types.campaign_error import CampaignErrorEnum
from google.ads.googleads.v13.errors.types.campaign_experiment_error import (
    CampaignExperimentErrorEnum,
)
from google.ads.googleads.v13.errors.types.campaign_feed_error import (
    CampaignFeedErrorEnum,
)
from google.ads.googleads.v13.errors.types.campaign_shared_set_error import (
    CampaignSharedSetErrorEnum,
)
from google.ads.googleads.v13.errors.types.change_event_error import (
    ChangeEventErrorEnum,
)
from google.ads.googleads.v13.errors.types.change_status_error import (
    ChangeStatusErrorEnum,
)
from google.ads.googleads.v13.errors.types.collection_size_error import (
    CollectionSizeErrorEnum,
)
from google.ads.googleads.v13.errors.types.context_error import ContextErrorEnum
from google.ads.googleads.v13.errors.types.conversion_action_error import (
    ConversionActionErrorEnum,
)
from google.ads.googleads.v13.errors.types.conversion_adjustment_upload_error import (
    ConversionAdjustmentUploadErrorEnum,
)
from google.ads.googleads.v13.errors.types.conversion_custom_variable_error import (
    ConversionCustomVariableErrorEnum,
)
from google.ads.googleads.v13.errors.types.conversion_goal_campaign_config_error import (
    ConversionGoalCampaignConfigErrorEnum,
)
from google.ads.googleads.v13.errors.types.conversion_upload_error import (
    ConversionUploadErrorEnum,
)
from google.ads.googleads.v13.errors.types.conversion_value_rule_error import (
    ConversionValueRuleErrorEnum,
)
from google.ads.googleads.v13.errors.types.conversion_value_rule_set_error import (
    ConversionValueRuleSetErrorEnum,
)
from google.ads.googleads.v13.errors.types.country_code_error import (
    CountryCodeErrorEnum,
)
from google.ads.googleads.v13.errors.types.criterion_error import CriterionErrorEnum
from google.ads.googleads.v13.errors.types.currency_code_error import (
    CurrencyCodeErrorEnum,
)
from google.ads.googleads.v13.errors.types.currency_error import CurrencyErrorEnum
from google.ads.googleads.v13.errors.types.custom_audience_error import (
    CustomAudienceErrorEnum,
)
from google.ads.googleads.v13.errors.types.custom_conversion_goal_error import (
    CustomConversionGoalErrorEnum,
)
from google.ads.googleads.v13.errors.types.custom_interest_error import (
    CustomInterestErrorEnum,
)
from google.ads.googleads.v13.errors.types.customer_client_link_error import (
    CustomerClientLinkErrorEnum,
)
from google.ads.googleads.v13.errors.types.customer_customizer_error import (
    CustomerCustomizerErrorEnum,
)
from google.ads.googleads.v13.errors.types.customer_error import CustomerErrorEnum
from google.ads.googleads.v13.errors.types.customer_feed_error import (
    CustomerFeedErrorEnum,
)
from google.ads.googleads.v13.errors.types.customer_manager_link_error import (
    CustomerManagerLinkErrorEnum,
)
from google.ads.googleads.v13.errors.types.customer_sk_ad_network_conversion_value_schema_error import (
    CustomerSkAdNetworkConversionValueSchemaErrorEnum,
)
from google.ads.googleads.v13.errors.types.customer_user_access_error import (
    CustomerUserAccessErrorEnum,
)
from google.ads.googleads.v13.errors.types.customizer_attribute_error import (
    CustomizerAttributeErrorEnum,
)
from google.ads.googleads.v13.errors.types.database_error import DatabaseErrorEnum
from google.ads.googleads.v13.errors.types.date_error import DateErrorEnum
from google.ads.googleads.v13.errors.types.date_range_error import DateRangeErrorEnum
from google.ads.googleads.v13.errors.types.distinct_error import DistinctErrorEnum
from google.ads.googleads.v13.errors.types.enum_error import EnumErrorEnum
from google.ads.googleads.v13.errors.types.experiment_arm_error import (
    ExperimentArmErrorEnum,
)
from google.ads.googleads.v13.errors.types.experiment_error import ExperimentErrorEnum
from google.ads.googleads.v13.errors.types.extension_feed_item_error import (
    ExtensionFeedItemErrorEnum,
)
from google.ads.googleads.v13.errors.types.extension_setting_error import (
    ExtensionSettingErrorEnum,
)
from google.ads.googleads.v13.errors.types.feed_attribute_reference_error import (
    FeedAttributeReferenceErrorEnum,
)
from google.ads.googleads.v13.errors.types.feed_error import FeedErrorEnum
from google.ads.googleads.v13.errors.types.feed_item_error import FeedItemErrorEnum
from google.ads.googleads.v13.errors.types.feed_item_set_error import (
    FeedItemSetErrorEnum,
)
from google.ads.googleads.v13.errors.types.feed_item_set_link_error import (
    FeedItemSetLinkErrorEnum,
)
from google.ads.googleads.v13.errors.types.feed_item_target_error import (
    FeedItemTargetErrorEnum,
)
from google.ads.googleads.v13.errors.types.feed_item_validation_error import (
    FeedItemValidationErrorEnum,
)
from google.ads.googleads.v13.errors.types.feed_mapping_error import (
    FeedMappingErrorEnum,
)
from google.ads.googleads.v13.errors.types.field_error import FieldErrorEnum
from google.ads.googleads.v13.errors.types.field_mask_error import FieldMaskErrorEnum
from google.ads.googleads.v13.errors.types.function_error import FunctionErrorEnum
from google.ads.googleads.v13.errors.types.function_parsing_error import (
    FunctionParsingErrorEnum,
)
from google.ads.googleads.v13.errors.types.geo_target_constant_suggestion_error import (
    GeoTargetConstantSuggestionErrorEnum,
)
from google.ads.googleads.v13.errors.types.header_error import HeaderErrorEnum
from google.ads.googleads.v13.errors.types.id_error import IdErrorEnum
from google.ads.googleads.v13.errors.types.image_error import ImageErrorEnum
from google.ads.googleads.v13.errors.types.internal_error import InternalErrorEnum
from google.ads.googleads.v13.errors.types.invoice_error import InvoiceErrorEnum
from google.ads.googleads.v13.errors.types.keyword_plan_ad_group_error import (
    KeywordPlanAdGroupErrorEnum,
)
from google.ads.googleads.v13.errors.types.keyword_plan_ad_group_keyword_error import (
    KeywordPlanAdGroupKeywordErrorEnum,
)
from google.ads.googleads.v13.errors.types.keyword_plan_campaign_error import (
    KeywordPlanCampaignErrorEnum,
)
from google.ads.googleads.v13.errors.types.keyword_plan_campaign_keyword_error import (
    KeywordPlanCampaignKeywordErrorEnum,
)
from google.ads.googleads.v13.errors.types.keyword_plan_error import (
    KeywordPlanErrorEnum,
)
from google.ads.googleads.v13.errors.types.keyword_plan_idea_error import (
    KeywordPlanIdeaErrorEnum,
)
from google.ads.googleads.v13.errors.types.label_error import LabelErrorEnum
from google.ads.googleads.v13.errors.types.language_code_error import (
    LanguageCodeErrorEnum,
)
from google.ads.googleads.v13.errors.types.list_operation_error import (
    ListOperationErrorEnum,
)
from google.ads.googleads.v13.errors.types.manager_link_error import (
    ManagerLinkErrorEnum,
)
from google.ads.googleads.v13.errors.types.media_bundle_error import (
    MediaBundleErrorEnum,
)
from google.ads.googleads.v13.errors.types.media_file_error import MediaFileErrorEnum
from google.ads.googleads.v13.errors.types.media_upload_error import (
    MediaUploadErrorEnum,
)
from google.ads.googleads.v13.errors.types.merchant_center_error import (
    MerchantCenterErrorEnum,
)
from google.ads.googleads.v13.errors.types.multiplier_error import MultiplierErrorEnum
from google.ads.googleads.v13.errors.types.mutate_error import MutateErrorEnum
from google.ads.googleads.v13.errors.types.new_resource_creation_error import (
    NewResourceCreationErrorEnum,
)
from google.ads.googleads.v13.errors.types.not_allowlisted_error import (
    NotAllowlistedErrorEnum,
)
from google.ads.googleads.v13.errors.types.not_empty_error import NotEmptyErrorEnum
from google.ads.googleads.v13.errors.types.null_error import NullErrorEnum
from google.ads.googleads.v13.errors.types.offline_user_data_job_error import (
    OfflineUserDataJobErrorEnum,
)
from google.ads.googleads.v13.errors.types.operation_access_denied_error import (
    OperationAccessDeniedErrorEnum,
)
from google.ads.googleads.v13.errors.types.operator_error import OperatorErrorEnum
from google.ads.googleads.v13.errors.types.partial_failure_error import (
    PartialFailureErrorEnum,
)
from google.ads.googleads.v13.errors.types.payments_account_error import (
    PaymentsAccountErrorEnum,
)
from google.ads.googleads.v13.errors.types.policy_finding_error import (
    PolicyFindingErrorEnum,
)
from google.ads.googleads.v13.errors.types.policy_validation_parameter_error import (
    PolicyValidationParameterErrorEnum,
)
from google.ads.googleads.v13.errors.types.policy_violation_error import (
    PolicyViolationErrorEnum,
)
from google.ads.googleads.v13.errors.types.query_error import QueryErrorEnum
from google.ads.googleads.v13.errors.types.quota_error import QuotaErrorEnum
from google.ads.googleads.v13.errors.types.range_error import RangeErrorEnum
from google.ads.googleads.v13.errors.types.reach_plan_error import ReachPlanErrorEnum
from google.ads.googleads.v13.errors.types.recommendation_error import (
    RecommendationErrorEnum,
)
from google.ads.googleads.v13.errors.types.region_code_error import RegionCodeErrorEnum
from google.ads.googleads.v13.errors.types.request_error import RequestErrorEnum
from google.ads.googleads.v13.errors.types.resource_access_denied_error import (
    ResourceAccessDeniedErrorEnum,
)
from google.ads.googleads.v13.errors.types.resource_count_limit_exceeded_error import (
    ResourceCountLimitExceededErrorEnum,
)
from google.ads.googleads.v13.errors.types.setting_error import SettingErrorEnum
from google.ads.googleads.v13.errors.types.shared_criterion_error import (
    SharedCriterionErrorEnum,
)
from google.ads.googleads.v13.errors.types.shared_set_error import SharedSetErrorEnum
from google.ads.googleads.v13.errors.types.size_limit_error import SizeLimitErrorEnum
from google.ads.googleads.v13.errors.types.smart_campaign_error import (
    SmartCampaignErrorEnum,
)
from google.ads.googleads.v13.errors.types.string_format_error import (
    StringFormatErrorEnum,
)
from google.ads.googleads.v13.errors.types.string_length_error import (
    StringLengthErrorEnum,
)
from google.ads.googleads.v13.errors.types.third_party_app_analytics_link_error import (
    ThirdPartyAppAnalyticsLinkErrorEnum,
)
from google.ads.googleads.v13.errors.types.time_zone_error import TimeZoneErrorEnum
from google.ads.googleads.v13.errors.types.url_field_error import UrlFieldErrorEnum
from google.ads.googleads.v13.errors.types.user_data_error import UserDataErrorEnum
from google.ads.googleads.v13.errors.types.user_list_error import UserListErrorEnum
from google.ads.googleads.v13.errors.types.youtube_video_registration_error import (
    YoutubeVideoRegistrationErrorEnum,
)

_M = TypeVar("_M")

class ErrorCode(proto.Message):
    request_error: RequestErrorEnum.RequestError
    bidding_strategy_error: BiddingStrategyErrorEnum.BiddingStrategyError
    url_field_error: UrlFieldErrorEnum.UrlFieldError
    list_operation_error: ListOperationErrorEnum.ListOperationError
    query_error: QueryErrorEnum.QueryError
    mutate_error: MutateErrorEnum.MutateError
    field_mask_error: FieldMaskErrorEnum.FieldMaskError
    authorization_error: AuthorizationErrorEnum.AuthorizationError
    internal_error: InternalErrorEnum.InternalError
    quota_error: QuotaErrorEnum.QuotaError
    ad_error: AdErrorEnum.AdError
    ad_group_error: AdGroupErrorEnum.AdGroupError
    campaign_budget_error: CampaignBudgetErrorEnum.CampaignBudgetError
    campaign_error: CampaignErrorEnum.CampaignError
    authentication_error: AuthenticationErrorEnum.AuthenticationError
    ad_group_criterion_customizer_error: AdGroupCriterionCustomizerErrorEnum.AdGroupCriterionCustomizerError
    ad_group_criterion_error: AdGroupCriterionErrorEnum.AdGroupCriterionError
    ad_group_customizer_error: AdGroupCustomizerErrorEnum.AdGroupCustomizerError
    ad_customizer_error: AdCustomizerErrorEnum.AdCustomizerError
    ad_group_ad_error: AdGroupAdErrorEnum.AdGroupAdError
    ad_sharing_error: AdSharingErrorEnum.AdSharingError
    adx_error: AdxErrorEnum.AdxError
    asset_error: AssetErrorEnum.AssetError
    asset_group_asset_error: AssetGroupAssetErrorEnum.AssetGroupAssetError
    asset_group_listing_group_filter_error: AssetGroupListingGroupFilterErrorEnum.AssetGroupListingGroupFilterError
    asset_group_error: AssetGroupErrorEnum.AssetGroupError
    asset_set_asset_error: AssetSetAssetErrorEnum.AssetSetAssetError
    asset_set_link_error: AssetSetLinkErrorEnum.AssetSetLinkError
    asset_set_error: AssetSetErrorEnum.AssetSetError
    bidding_error: BiddingErrorEnum.BiddingError
    campaign_criterion_error: CampaignCriterionErrorEnum.CampaignCriterionError
    campaign_conversion_goal_error: CampaignConversionGoalErrorEnum.CampaignConversionGoalError
    campaign_customizer_error: CampaignCustomizerErrorEnum.CampaignCustomizerError
    collection_size_error: CollectionSizeErrorEnum.CollectionSizeError
    conversion_goal_campaign_config_error: ConversionGoalCampaignConfigErrorEnum.ConversionGoalCampaignConfigError
    country_code_error: CountryCodeErrorEnum.CountryCodeError
    criterion_error: CriterionErrorEnum.CriterionError
    custom_conversion_goal_error: CustomConversionGoalErrorEnum.CustomConversionGoalError
    customer_customizer_error: CustomerCustomizerErrorEnum.CustomerCustomizerError
    customer_error: CustomerErrorEnum.CustomerError
    customizer_attribute_error: CustomizerAttributeErrorEnum.CustomizerAttributeError
    date_error: DateErrorEnum.DateError
    date_range_error: DateRangeErrorEnum.DateRangeError
    distinct_error: DistinctErrorEnum.DistinctError
    feed_attribute_reference_error: FeedAttributeReferenceErrorEnum.FeedAttributeReferenceError
    function_error: FunctionErrorEnum.FunctionError
    function_parsing_error: FunctionParsingErrorEnum.FunctionParsingError
    id_error: IdErrorEnum.IdError
    image_error: ImageErrorEnum.ImageError
    language_code_error: LanguageCodeErrorEnum.LanguageCodeError
    media_bundle_error: MediaBundleErrorEnum.MediaBundleError
    media_upload_error: MediaUploadErrorEnum.MediaUploadError
    media_file_error: MediaFileErrorEnum.MediaFileError
    merchant_center_error: MerchantCenterErrorEnum.MerchantCenterError
    multiplier_error: MultiplierErrorEnum.MultiplierError
    new_resource_creation_error: NewResourceCreationErrorEnum.NewResourceCreationError
    not_empty_error: NotEmptyErrorEnum.NotEmptyError
    null_error: NullErrorEnum.NullError
    operator_error: OperatorErrorEnum.OperatorError
    range_error: RangeErrorEnum.RangeError
    recommendation_error: RecommendationErrorEnum.RecommendationError
    region_code_error: RegionCodeErrorEnum.RegionCodeError
    setting_error: SettingErrorEnum.SettingError
    string_format_error: StringFormatErrorEnum.StringFormatError
    string_length_error: StringLengthErrorEnum.StringLengthError
    operation_access_denied_error: OperationAccessDeniedErrorEnum.OperationAccessDeniedError
    resource_access_denied_error: ResourceAccessDeniedErrorEnum.ResourceAccessDeniedError
    resource_count_limit_exceeded_error: ResourceCountLimitExceededErrorEnum.ResourceCountLimitExceededError
    youtube_video_registration_error: YoutubeVideoRegistrationErrorEnum.YoutubeVideoRegistrationError
    ad_group_bid_modifier_error: AdGroupBidModifierErrorEnum.AdGroupBidModifierError
    context_error: ContextErrorEnum.ContextError
    field_error: FieldErrorEnum.FieldError
    shared_set_error: SharedSetErrorEnum.SharedSetError
    shared_criterion_error: SharedCriterionErrorEnum.SharedCriterionError
    campaign_shared_set_error: CampaignSharedSetErrorEnum.CampaignSharedSetError
    conversion_action_error: ConversionActionErrorEnum.ConversionActionError
    conversion_adjustment_upload_error: ConversionAdjustmentUploadErrorEnum.ConversionAdjustmentUploadError
    conversion_custom_variable_error: ConversionCustomVariableErrorEnum.ConversionCustomVariableError
    conversion_upload_error: ConversionUploadErrorEnum.ConversionUploadError
    conversion_value_rule_error: ConversionValueRuleErrorEnum.ConversionValueRuleError
    conversion_value_rule_set_error: ConversionValueRuleSetErrorEnum.ConversionValueRuleSetError
    header_error: HeaderErrorEnum.HeaderError
    database_error: DatabaseErrorEnum.DatabaseError
    policy_finding_error: PolicyFindingErrorEnum.PolicyFindingError
    enum_error: EnumErrorEnum.EnumError
    keyword_plan_error: KeywordPlanErrorEnum.KeywordPlanError
    keyword_plan_campaign_error: KeywordPlanCampaignErrorEnum.KeywordPlanCampaignError
    keyword_plan_campaign_keyword_error: KeywordPlanCampaignKeywordErrorEnum.KeywordPlanCampaignKeywordError
    keyword_plan_ad_group_error: KeywordPlanAdGroupErrorEnum.KeywordPlanAdGroupError
    keyword_plan_ad_group_keyword_error: KeywordPlanAdGroupKeywordErrorEnum.KeywordPlanAdGroupKeywordError
    keyword_plan_idea_error: KeywordPlanIdeaErrorEnum.KeywordPlanIdeaError
    account_budget_proposal_error: AccountBudgetProposalErrorEnum.AccountBudgetProposalError
    user_list_error: UserListErrorEnum.UserListError
    change_event_error: ChangeEventErrorEnum.ChangeEventError
    change_status_error: ChangeStatusErrorEnum.ChangeStatusError
    feed_error: FeedErrorEnum.FeedError
    geo_target_constant_suggestion_error: GeoTargetConstantSuggestionErrorEnum.GeoTargetConstantSuggestionError
    campaign_draft_error: CampaignDraftErrorEnum.CampaignDraftError
    feed_item_error: FeedItemErrorEnum.FeedItemError
    label_error: LabelErrorEnum.LabelError
    billing_setup_error: BillingSetupErrorEnum.BillingSetupError
    customer_client_link_error: CustomerClientLinkErrorEnum.CustomerClientLinkError
    customer_manager_link_error: CustomerManagerLinkErrorEnum.CustomerManagerLinkError
    feed_mapping_error: FeedMappingErrorEnum.FeedMappingError
    customer_feed_error: CustomerFeedErrorEnum.CustomerFeedError
    ad_group_feed_error: AdGroupFeedErrorEnum.AdGroupFeedError
    campaign_feed_error: CampaignFeedErrorEnum.CampaignFeedError
    custom_interest_error: CustomInterestErrorEnum.CustomInterestError
    campaign_experiment_error: CampaignExperimentErrorEnum.CampaignExperimentError
    extension_feed_item_error: ExtensionFeedItemErrorEnum.ExtensionFeedItemError
    ad_parameter_error: AdParameterErrorEnum.AdParameterError
    feed_item_validation_error: FeedItemValidationErrorEnum.FeedItemValidationError
    extension_setting_error: ExtensionSettingErrorEnum.ExtensionSettingError
    feed_item_set_error: FeedItemSetErrorEnum.FeedItemSetError
    feed_item_set_link_error: FeedItemSetLinkErrorEnum.FeedItemSetLinkError
    feed_item_target_error: FeedItemTargetErrorEnum.FeedItemTargetError
    policy_violation_error: PolicyViolationErrorEnum.PolicyViolationError
    partial_failure_error: PartialFailureErrorEnum.PartialFailureError
    policy_validation_parameter_error: PolicyValidationParameterErrorEnum.PolicyValidationParameterError
    size_limit_error: SizeLimitErrorEnum.SizeLimitError
    offline_user_data_job_error: OfflineUserDataJobErrorEnum.OfflineUserDataJobError
    not_allowlisted_error: NotAllowlistedErrorEnum.NotAllowlistedError
    manager_link_error: ManagerLinkErrorEnum.ManagerLinkError
    currency_code_error: CurrencyCodeErrorEnum.CurrencyCodeError
    experiment_error: ExperimentErrorEnum.ExperimentError
    access_invitation_error: AccessInvitationErrorEnum.AccessInvitationError
    reach_plan_error: ReachPlanErrorEnum.ReachPlanError
    invoice_error: InvoiceErrorEnum.InvoiceError
    payments_account_error: PaymentsAccountErrorEnum.PaymentsAccountError
    time_zone_error: TimeZoneErrorEnum.TimeZoneError
    asset_link_error: AssetLinkErrorEnum.AssetLinkError
    user_data_error: UserDataErrorEnum.UserDataError
    batch_job_error: BatchJobErrorEnum.BatchJobError
    account_link_error: AccountLinkErrorEnum.AccountLinkError
    third_party_app_analytics_link_error: ThirdPartyAppAnalyticsLinkErrorEnum.ThirdPartyAppAnalyticsLinkError
    customer_user_access_error: CustomerUserAccessErrorEnum.CustomerUserAccessError
    custom_audience_error: CustomAudienceErrorEnum.CustomAudienceError
    audience_error: AudienceErrorEnum.AudienceError
    smart_campaign_error: SmartCampaignErrorEnum.SmartCampaignError
    experiment_arm_error: ExperimentArmErrorEnum.ExperimentArmError
    audience_insights_error: AudienceInsightsErrorEnum.AudienceInsightsError
    customer_sk_ad_network_conversion_value_schema_error: CustomerSkAdNetworkConversionValueSchemaErrorEnum.CustomerSkAdNetworkConversionValueSchemaError
    currency_error: CurrencyErrorEnum.CurrencyError
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        request_error: RequestErrorEnum.RequestError = ...,
        bidding_strategy_error: BiddingStrategyErrorEnum.BiddingStrategyError = ...,
        url_field_error: UrlFieldErrorEnum.UrlFieldError = ...,
        list_operation_error: ListOperationErrorEnum.ListOperationError = ...,
        query_error: QueryErrorEnum.QueryError = ...,
        mutate_error: MutateErrorEnum.MutateError = ...,
        field_mask_error: FieldMaskErrorEnum.FieldMaskError = ...,
        authorization_error: AuthorizationErrorEnum.AuthorizationError = ...,
        internal_error: InternalErrorEnum.InternalError = ...,
        quota_error: QuotaErrorEnum.QuotaError = ...,
        ad_error: AdErrorEnum.AdError = ...,
        ad_group_error: AdGroupErrorEnum.AdGroupError = ...,
        campaign_budget_error: CampaignBudgetErrorEnum.CampaignBudgetError = ...,
        campaign_error: CampaignErrorEnum.CampaignError = ...,
        authentication_error: AuthenticationErrorEnum.AuthenticationError = ...,
        ad_group_criterion_customizer_error: AdGroupCriterionCustomizerErrorEnum.AdGroupCriterionCustomizerError = ...,
        ad_group_criterion_error: AdGroupCriterionErrorEnum.AdGroupCriterionError = ...,
        ad_group_customizer_error: AdGroupCustomizerErrorEnum.AdGroupCustomizerError = ...,
        ad_customizer_error: AdCustomizerErrorEnum.AdCustomizerError = ...,
        ad_group_ad_error: AdGroupAdErrorEnum.AdGroupAdError = ...,
        ad_sharing_error: AdSharingErrorEnum.AdSharingError = ...,
        adx_error: AdxErrorEnum.AdxError = ...,
        asset_error: AssetErrorEnum.AssetError = ...,
        asset_group_asset_error: AssetGroupAssetErrorEnum.AssetGroupAssetError = ...,
        asset_group_listing_group_filter_error: AssetGroupListingGroupFilterErrorEnum.AssetGroupListingGroupFilterError = ...,
        asset_group_error: AssetGroupErrorEnum.AssetGroupError = ...,
        asset_set_asset_error: AssetSetAssetErrorEnum.AssetSetAssetError = ...,
        asset_set_link_error: AssetSetLinkErrorEnum.AssetSetLinkError = ...,
        asset_set_error: AssetSetErrorEnum.AssetSetError = ...,
        bidding_error: BiddingErrorEnum.BiddingError = ...,
        campaign_criterion_error: CampaignCriterionErrorEnum.CampaignCriterionError = ...,
        campaign_conversion_goal_error: CampaignConversionGoalErrorEnum.CampaignConversionGoalError = ...,
        campaign_customizer_error: CampaignCustomizerErrorEnum.CampaignCustomizerError = ...,
        collection_size_error: CollectionSizeErrorEnum.CollectionSizeError = ...,
        conversion_goal_campaign_config_error: ConversionGoalCampaignConfigErrorEnum.ConversionGoalCampaignConfigError = ...,
        country_code_error: CountryCodeErrorEnum.CountryCodeError = ...,
        criterion_error: CriterionErrorEnum.CriterionError = ...,
        custom_conversion_goal_error: CustomConversionGoalErrorEnum.CustomConversionGoalError = ...,
        customer_customizer_error: CustomerCustomizerErrorEnum.CustomerCustomizerError = ...,
        customer_error: CustomerErrorEnum.CustomerError = ...,
        customizer_attribute_error: CustomizerAttributeErrorEnum.CustomizerAttributeError = ...,
        date_error: DateErrorEnum.DateError = ...,
        date_range_error: DateRangeErrorEnum.DateRangeError = ...,
        distinct_error: DistinctErrorEnum.DistinctError = ...,
        feed_attribute_reference_error: FeedAttributeReferenceErrorEnum.FeedAttributeReferenceError = ...,
        function_error: FunctionErrorEnum.FunctionError = ...,
        function_parsing_error: FunctionParsingErrorEnum.FunctionParsingError = ...,
        id_error: IdErrorEnum.IdError = ...,
        image_error: ImageErrorEnum.ImageError = ...,
        language_code_error: LanguageCodeErrorEnum.LanguageCodeError = ...,
        media_bundle_error: MediaBundleErrorEnum.MediaBundleError = ...,
        media_upload_error: MediaUploadErrorEnum.MediaUploadError = ...,
        media_file_error: MediaFileErrorEnum.MediaFileError = ...,
        merchant_center_error: MerchantCenterErrorEnum.MerchantCenterError = ...,
        multiplier_error: MultiplierErrorEnum.MultiplierError = ...,
        new_resource_creation_error: NewResourceCreationErrorEnum.NewResourceCreationError = ...,
        not_empty_error: NotEmptyErrorEnum.NotEmptyError = ...,
        null_error: NullErrorEnum.NullError = ...,
        operator_error: OperatorErrorEnum.OperatorError = ...,
        range_error: RangeErrorEnum.RangeError = ...,
        recommendation_error: RecommendationErrorEnum.RecommendationError = ...,
        region_code_error: RegionCodeErrorEnum.RegionCodeError = ...,
        setting_error: SettingErrorEnum.SettingError = ...,
        string_format_error: StringFormatErrorEnum.StringFormatError = ...,
        string_length_error: StringLengthErrorEnum.StringLengthError = ...,
        operation_access_denied_error: OperationAccessDeniedErrorEnum.OperationAccessDeniedError = ...,
        resource_access_denied_error: ResourceAccessDeniedErrorEnum.ResourceAccessDeniedError = ...,
        resource_count_limit_exceeded_error: ResourceCountLimitExceededErrorEnum.ResourceCountLimitExceededError = ...,
        youtube_video_registration_error: YoutubeVideoRegistrationErrorEnum.YoutubeVideoRegistrationError = ...,
        ad_group_bid_modifier_error: AdGroupBidModifierErrorEnum.AdGroupBidModifierError = ...,
        context_error: ContextErrorEnum.ContextError = ...,
        field_error: FieldErrorEnum.FieldError = ...,
        shared_set_error: SharedSetErrorEnum.SharedSetError = ...,
        shared_criterion_error: SharedCriterionErrorEnum.SharedCriterionError = ...,
        campaign_shared_set_error: CampaignSharedSetErrorEnum.CampaignSharedSetError = ...,
        conversion_action_error: ConversionActionErrorEnum.ConversionActionError = ...,
        conversion_adjustment_upload_error: ConversionAdjustmentUploadErrorEnum.ConversionAdjustmentUploadError = ...,
        conversion_custom_variable_error: ConversionCustomVariableErrorEnum.ConversionCustomVariableError = ...,
        conversion_upload_error: ConversionUploadErrorEnum.ConversionUploadError = ...,
        conversion_value_rule_error: ConversionValueRuleErrorEnum.ConversionValueRuleError = ...,
        conversion_value_rule_set_error: ConversionValueRuleSetErrorEnum.ConversionValueRuleSetError = ...,
        header_error: HeaderErrorEnum.HeaderError = ...,
        database_error: DatabaseErrorEnum.DatabaseError = ...,
        policy_finding_error: PolicyFindingErrorEnum.PolicyFindingError = ...,
        enum_error: EnumErrorEnum.EnumError = ...,
        keyword_plan_error: KeywordPlanErrorEnum.KeywordPlanError = ...,
        keyword_plan_campaign_error: KeywordPlanCampaignErrorEnum.KeywordPlanCampaignError = ...,
        keyword_plan_campaign_keyword_error: KeywordPlanCampaignKeywordErrorEnum.KeywordPlanCampaignKeywordError = ...,
        keyword_plan_ad_group_error: KeywordPlanAdGroupErrorEnum.KeywordPlanAdGroupError = ...,
        keyword_plan_ad_group_keyword_error: KeywordPlanAdGroupKeywordErrorEnum.KeywordPlanAdGroupKeywordError = ...,
        keyword_plan_idea_error: KeywordPlanIdeaErrorEnum.KeywordPlanIdeaError = ...,
        account_budget_proposal_error: AccountBudgetProposalErrorEnum.AccountBudgetProposalError = ...,
        user_list_error: UserListErrorEnum.UserListError = ...,
        change_event_error: ChangeEventErrorEnum.ChangeEventError = ...,
        change_status_error: ChangeStatusErrorEnum.ChangeStatusError = ...,
        feed_error: FeedErrorEnum.FeedError = ...,
        geo_target_constant_suggestion_error: GeoTargetConstantSuggestionErrorEnum.GeoTargetConstantSuggestionError = ...,
        campaign_draft_error: CampaignDraftErrorEnum.CampaignDraftError = ...,
        feed_item_error: FeedItemErrorEnum.FeedItemError = ...,
        label_error: LabelErrorEnum.LabelError = ...,
        billing_setup_error: BillingSetupErrorEnum.BillingSetupError = ...,
        customer_client_link_error: CustomerClientLinkErrorEnum.CustomerClientLinkError = ...,
        customer_manager_link_error: CustomerManagerLinkErrorEnum.CustomerManagerLinkError = ...,
        feed_mapping_error: FeedMappingErrorEnum.FeedMappingError = ...,
        customer_feed_error: CustomerFeedErrorEnum.CustomerFeedError = ...,
        ad_group_feed_error: AdGroupFeedErrorEnum.AdGroupFeedError = ...,
        campaign_feed_error: CampaignFeedErrorEnum.CampaignFeedError = ...,
        custom_interest_error: CustomInterestErrorEnum.CustomInterestError = ...,
        campaign_experiment_error: CampaignExperimentErrorEnum.CampaignExperimentError = ...,
        extension_feed_item_error: ExtensionFeedItemErrorEnum.ExtensionFeedItemError = ...,
        ad_parameter_error: AdParameterErrorEnum.AdParameterError = ...,
        feed_item_validation_error: FeedItemValidationErrorEnum.FeedItemValidationError = ...,
        extension_setting_error: ExtensionSettingErrorEnum.ExtensionSettingError = ...,
        feed_item_set_error: FeedItemSetErrorEnum.FeedItemSetError = ...,
        feed_item_set_link_error: FeedItemSetLinkErrorEnum.FeedItemSetLinkError = ...,
        feed_item_target_error: FeedItemTargetErrorEnum.FeedItemTargetError = ...,
        policy_violation_error: PolicyViolationErrorEnum.PolicyViolationError = ...,
        partial_failure_error: PartialFailureErrorEnum.PartialFailureError = ...,
        policy_validation_parameter_error: PolicyValidationParameterErrorEnum.PolicyValidationParameterError = ...,
        size_limit_error: SizeLimitErrorEnum.SizeLimitError = ...,
        offline_user_data_job_error: OfflineUserDataJobErrorEnum.OfflineUserDataJobError = ...,
        not_allowlisted_error: NotAllowlistedErrorEnum.NotAllowlistedError = ...,
        manager_link_error: ManagerLinkErrorEnum.ManagerLinkError = ...,
        currency_code_error: CurrencyCodeErrorEnum.CurrencyCodeError = ...,
        experiment_error: ExperimentErrorEnum.ExperimentError = ...,
        access_invitation_error: AccessInvitationErrorEnum.AccessInvitationError = ...,
        reach_plan_error: ReachPlanErrorEnum.ReachPlanError = ...,
        invoice_error: InvoiceErrorEnum.InvoiceError = ...,
        payments_account_error: PaymentsAccountErrorEnum.PaymentsAccountError = ...,
        time_zone_error: TimeZoneErrorEnum.TimeZoneError = ...,
        asset_link_error: AssetLinkErrorEnum.AssetLinkError = ...,
        user_data_error: UserDataErrorEnum.UserDataError = ...,
        batch_job_error: BatchJobErrorEnum.BatchJobError = ...,
        account_link_error: AccountLinkErrorEnum.AccountLinkError = ...,
        third_party_app_analytics_link_error: ThirdPartyAppAnalyticsLinkErrorEnum.ThirdPartyAppAnalyticsLinkError = ...,
        customer_user_access_error: CustomerUserAccessErrorEnum.CustomerUserAccessError = ...,
        custom_audience_error: CustomAudienceErrorEnum.CustomAudienceError = ...,
        audience_error: AudienceErrorEnum.AudienceError = ...,
        smart_campaign_error: SmartCampaignErrorEnum.SmartCampaignError = ...,
        experiment_arm_error: ExperimentArmErrorEnum.ExperimentArmError = ...,
        audience_insights_error: AudienceInsightsErrorEnum.AudienceInsightsError = ...,
        customer_sk_ad_network_conversion_value_schema_error: CustomerSkAdNetworkConversionValueSchemaErrorEnum.CustomerSkAdNetworkConversionValueSchemaError = ...,
        currency_error: CurrencyErrorEnum.CurrencyError = ...
    ) -> None: ...

class ErrorDetails(proto.Message):
    unpublished_error_code: str
    policy_violation_details: PolicyViolationDetails
    policy_finding_details: PolicyFindingDetails
    quota_error_details: QuotaErrorDetails
    resource_count_details: ResourceCountDetails
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        unpublished_error_code: str = ...,
        policy_violation_details: PolicyViolationDetails = ...,
        policy_finding_details: PolicyFindingDetails = ...,
        quota_error_details: QuotaErrorDetails = ...,
        resource_count_details: ResourceCountDetails = ...
    ) -> None: ...

class ErrorLocation(proto.Message):
    class FieldPathElement(proto.Message):
        field_name: str
        index: int
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            field_name: str = ...,
            index: int = ...
        ) -> None: ...
    field_path_elements: MutableSequence[ErrorLocation.FieldPathElement]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        field_path_elements: MutableSequence[ErrorLocation.FieldPathElement] = ...
    ) -> None: ...

class GoogleAdsError(proto.Message):
    error_code: ErrorCode
    message: str
    trigger: Value
    location: ErrorLocation
    details: ErrorDetails
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        error_code: ErrorCode = ...,
        message: str = ...,
        trigger: Value = ...,
        location: ErrorLocation = ...,
        details: ErrorDetails = ...
    ) -> None: ...

class GoogleAdsFailure(proto.Message):
    errors: MutableSequence[GoogleAdsError]
    request_id: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        errors: MutableSequence[GoogleAdsError] = ...,
        request_id: str = ...
    ) -> None: ...

class PolicyFindingDetails(proto.Message):
    policy_topic_entries: MutableSequence[PolicyTopicEntry]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        policy_topic_entries: MutableSequence[PolicyTopicEntry] = ...
    ) -> None: ...

class PolicyViolationDetails(proto.Message):
    external_policy_description: str
    key: PolicyViolationKey
    external_policy_name: str
    is_exemptible: bool
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        external_policy_description: str = ...,
        key: PolicyViolationKey = ...,
        external_policy_name: str = ...,
        is_exemptible: bool = ...
    ) -> None: ...

class QuotaErrorDetails(proto.Message):
    class QuotaRateScope(proto.Enum):
        UNSPECIFIED = 0
        UNKNOWN = 1
        ACCOUNT = 2
        DEVELOPER = 3
    rate_scope: QuotaErrorDetails.QuotaRateScope
    rate_name: str
    retry_delay: Duration
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        rate_scope: QuotaErrorDetails.QuotaRateScope = ...,
        rate_name: str = ...,
        retry_delay: Duration = ...
    ) -> None: ...

class ResourceCountDetails(proto.Message):
    enclosing_id: str
    enclosing_resource: str
    limit: int
    limit_type: ResourceLimitTypeEnum.ResourceLimitType
    existing_count: int
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        enclosing_id: str = ...,
        enclosing_resource: str = ...,
        limit: int = ...,
        limit_type: ResourceLimitTypeEnum.ResourceLimitType = ...,
        existing_count: int = ...
    ) -> None: ...



from collections.abc import Mapping
from typing import Any, TypeVar

import google.protobuf.message
import proto

_M = TypeVar("_M")

class FeedItemQualityDisapprovalReasonEnum(proto.Message):
    class FeedItemQualityDisapprovalReason(proto.Enum):
        UNSPECIFIED = 0
        UNKNOWN = 1
        PRICE_TABLE_REPETITIVE_HEADERS = 2
        PRICE_TABLE_REPETITIVE_DESCRIPTION = 3
        PRICE_TABLE_INCONSISTENT_ROWS = 4
        PRICE_DESCRIPTION_HAS_PRICE_QUALIFIERS = 5
        PRICE_UNSUPPORTED_LANGUAGE = 6
        PRICE_TABLE_ROW_HEADER_TABLE_TYPE_MISMATCH = 7
        PRICE_TABLE_ROW_HEADER_HAS_PROMOTIONAL_TEXT = 8
        PRICE_TABLE_ROW_DESCRIPTION_NOT_RELEVANT = 9
        PRICE_TABLE_ROW_DESCRIPTION_HAS_PROMOTIONAL_TEXT = 10
        PRICE_TABLE_ROW_HEADER_DESCRIPTION_REPETITIVE = 11
        PRICE_TABLE_ROW_UNRATEABLE = 12
        PRICE_TABLE_ROW_PRICE_INVALID = 13
        PRICE_TABLE_ROW_URL_INVALID = 14
        PRICE_HEADER_OR_DESCRIPTION_HAS_PRICE = 15
        STRUCTURED_SNIPPETS_HEADER_POLICY_VIOLATED = 16
        STRUCTURED_SNIPPETS_REPEATED_VALUES = 17
        STRUCTURED_SNIPPETS_EDITORIAL_GUIDELINES = 18
        STRUCTURED_SNIPPETS_HAS_PROMOTIONAL_TEXT = 19
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
    ) -> None: ...



from collections.abc import Mapping
from typing import Any, TypeVar

import google.protobuf.message
import proto

_M = TypeVar("_M")

class AuthenticationErrorEnum(proto.Message):
    class AuthenticationError(proto.Enum):
        UNSPECIFIED = 0
        UNKNOWN = 1
        AUTHENTICATION_ERROR = 2
        CLIENT_CUSTOMER_ID_INVALID = 5
        CUSTOMER_NOT_FOUND = 8
        GOOGLE_ACCOUNT_DELETED = 9
        GOOGLE_ACCOUNT_COOKIE_INVALID = 10
        GOOGLE_ACCOUNT_AUTHENTICATION_FAILED = 25
        GOOGLE_ACCOUNT_USER_AND_ADS_USER_MISMATCH = 12
        LOGIN_COOKIE_REQUIRED = 13
        NOT_ADS_USER = 14
        OAUTH_TOKEN_INVALID = 15
        OAUTH_TOKEN_EXPIRED = 16
        OAUTH_TOKEN_DISABLED = 17
        OAUTH_TOKEN_REVOKED = 18
        OAUTH_TOKEN_HEADER_INVALID = 19
        LOGIN_COOKIE_INVALID = 20
        USER_ID_INVALID = 22
        TWO_STEP_VERIFICATION_NOT_ENROLLED = 23
        ADVANCED_PROTECTION_NOT_ENROLLED = 24
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
    ) -> None: ...



from collections.abc import Mapping, MutableSequence
from typing import Any, TypeVar

import google.protobuf.message
import proto

from google.ads.googleads.v14.common.types.criteria import KeywordInfo
from google.ads.googleads.v14.enums.types.app_bidding_goal import AppBiddingGoalEnum
from google.ads.googleads.v14.enums.types.keyword_match_type import KeywordMatchTypeEnum
from google.ads.googleads.v14.enums.types.recommendation_type import (
    RecommendationTypeEnum,
)
from google.ads.googleads.v14.enums.types.shopping_add_products_to_campaign_recommendation_enum import (
    ShoppingAddProductsToCampaignRecommendationEnum,
)
from google.ads.googleads.v14.enums.types.target_cpa_opt_in_recommendation_goal import (
    TargetCpaOptInRecommendationGoalEnum,
)
from google.ads.googleads.v14.resources.types.ad import Ad
from google.ads.googleads.v14.resources.types.asset import Asset

_M = TypeVar("_M")

class Recommendation(proto.Message):
    class CallAssetRecommendation(proto.Message):
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
        ) -> None: ...
        ...

    class CalloutAssetRecommendation(proto.Message):
        recommended_campaign_callout_assets: MutableSequence[Asset]
        recommended_customer_callout_assets: MutableSequence[Asset]
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            recommended_campaign_callout_assets: MutableSequence[Asset] = ...,
            recommended_customer_callout_assets: MutableSequence[Asset] = ...,
        ) -> None: ...

    class CampaignBudget(proto.Message):
        current_amount_micros: int
        recommended_new_amount_micros: int
        new_start_date: str
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            current_amount_micros: int = ...,
            recommended_new_amount_micros: int = ...,
            new_start_date: str = ...,
        ) -> None: ...

    class CampaignBudgetRecommendation(proto.Message):
        class CampaignBudgetRecommendationOption(proto.Message):
            budget_amount_micros: int
            impact: Recommendation.RecommendationImpact
            def __init__(
                self: _M,
                mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
                *,
                ignore_unknown_fields: bool = ...,
                budget_amount_micros: int = ...,
                impact: Recommendation.RecommendationImpact = ...,
            ) -> None: ...
        current_budget_amount_micros: int
        recommended_budget_amount_micros: int
        budget_options: MutableSequence[
            Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption
        ]
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            current_budget_amount_micros: int = ...,
            recommended_budget_amount_micros: int = ...,
            budget_options: MutableSequence[
                Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption
            ] = ...,
        ) -> None: ...

    class DisplayExpansionOptInRecommendation(proto.Message):
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
        ) -> None: ...
        ...

    class DynamicImageExtensionOptInRecommendation(proto.Message):
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
        ) -> None: ...
        ...

    class EnhancedCpcOptInRecommendation(proto.Message):
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
        ) -> None: ...
        ...

    class ForecastingSetTargetRoasRecommendation(proto.Message):
        recommended_target_roas: float
        campaign_budget: Recommendation.CampaignBudget
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            recommended_target_roas: float = ...,
            campaign_budget: Recommendation.CampaignBudget = ...,
        ) -> None: ...

    class ImprovePerformanceMaxAdStrengthRecommendation(proto.Message):
        asset_group: str
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            asset_group: str = ...,
        ) -> None: ...

    class KeywordMatchTypeRecommendation(proto.Message):
        keyword: KeywordInfo
        recommended_match_type: KeywordMatchTypeEnum.KeywordMatchType
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            keyword: KeywordInfo = ...,
            recommended_match_type: KeywordMatchTypeEnum.KeywordMatchType = ...,
        ) -> None: ...

    class KeywordRecommendation(proto.Message):
        class SearchTerm(proto.Message):
            text: str
            estimated_weekly_search_count: int
            def __init__(
                self: _M,
                mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
                *,
                ignore_unknown_fields: bool = ...,
                text: str = ...,
                estimated_weekly_search_count: int = ...,
            ) -> None: ...
        keyword: KeywordInfo
        search_terms: MutableSequence[Recommendation.KeywordRecommendation.SearchTerm]
        recommended_cpc_bid_micros: int
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            keyword: KeywordInfo = ...,
            search_terms: MutableSequence[
                Recommendation.KeywordRecommendation.SearchTerm
            ] = ...,
            recommended_cpc_bid_micros: int = ...,
        ) -> None: ...

    class LowerTargetRoasRecommendation(proto.Message):
        target_adjustment: Recommendation.TargetAdjustmentInfo
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            target_adjustment: Recommendation.TargetAdjustmentInfo = ...,
        ) -> None: ...

    class MaximizeClicksOptInRecommendation(proto.Message):
        recommended_budget_amount_micros: int
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            recommended_budget_amount_micros: int = ...,
        ) -> None: ...

    class MaximizeConversionsOptInRecommendation(proto.Message):
        recommended_budget_amount_micros: int
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            recommended_budget_amount_micros: int = ...,
        ) -> None: ...

    class MerchantInfo(proto.Message):
        id: int
        name: str
        multi_client: bool
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            id: int = ...,
            name: str = ...,
            multi_client: bool = ...,
        ) -> None: ...

    class MigrateDynamicSearchAdsCampaignToPerformanceMaxRecommendation(proto.Message):
        apply_link: str
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            apply_link: str = ...,
        ) -> None: ...

    class MoveUnusedBudgetRecommendation(proto.Message):
        excess_campaign_budget: str
        budget_recommendation: Recommendation.CampaignBudgetRecommendation
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            excess_campaign_budget: str = ...,
            budget_recommendation: Recommendation.CampaignBudgetRecommendation = ...,
        ) -> None: ...

    class OptimizeAdRotationRecommendation(proto.Message):
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
        ) -> None: ...
        ...

    class PerformanceMaxOptInRecommendation(proto.Message):
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
        ) -> None: ...
        ...

    class RaiseTargetCpaBidTooLowRecommendation(proto.Message):
        recommended_target_multiplier: float
        average_target_cpa_micros: int
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            recommended_target_multiplier: float = ...,
            average_target_cpa_micros: int = ...,
        ) -> None: ...

    class RaiseTargetCpaRecommendation(proto.Message):
        target_adjustment: Recommendation.TargetAdjustmentInfo
        app_bidding_goal: AppBiddingGoalEnum.AppBiddingGoal
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            target_adjustment: Recommendation.TargetAdjustmentInfo = ...,
            app_bidding_goal: AppBiddingGoalEnum.AppBiddingGoal = ...,
        ) -> None: ...

    class RecommendationImpact(proto.Message):
        base_metrics: Recommendation.RecommendationMetrics
        potential_metrics: Recommendation.RecommendationMetrics
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            base_metrics: Recommendation.RecommendationMetrics = ...,
            potential_metrics: Recommendation.RecommendationMetrics = ...,
        ) -> None: ...

    class RecommendationMetrics(proto.Message):
        impressions: float
        clicks: float
        cost_micros: int
        conversions: float
        video_views: float
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            impressions: float = ...,
            clicks: float = ...,
            cost_micros: int = ...,
            conversions: float = ...,
            video_views: float = ...,
        ) -> None: ...

    class ResponsiveSearchAdAssetRecommendation(proto.Message):
        current_ad: Ad
        recommended_assets: Ad
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            current_ad: Ad = ...,
            recommended_assets: Ad = ...,
        ) -> None: ...

    class ResponsiveSearchAdImproveAdStrengthRecommendation(proto.Message):
        current_ad: Ad
        recommended_ad: Ad
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            current_ad: Ad = ...,
            recommended_ad: Ad = ...,
        ) -> None: ...

    class ResponsiveSearchAdRecommendation(proto.Message):
        ad: Ad
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            ad: Ad = ...,
        ) -> None: ...

    class SearchPartnersOptInRecommendation(proto.Message):
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
        ) -> None: ...
        ...

    class ShoppingAddProductsToCampaignRecommendation(proto.Message):
        merchant: Recommendation.MerchantInfo
        feed_label: str
        reason: ShoppingAddProductsToCampaignRecommendationEnum.Reason
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            merchant: Recommendation.MerchantInfo = ...,
            feed_label: str = ...,
            reason: ShoppingAddProductsToCampaignRecommendationEnum.Reason = ...,
        ) -> None: ...

    class ShoppingFixDisapprovedProductsRecommendation(proto.Message):
        merchant: Recommendation.MerchantInfo
        feed_label: str
        products_count: int
        disapproved_products_count: int
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            merchant: Recommendation.MerchantInfo = ...,
            feed_label: str = ...,
            products_count: int = ...,
            disapproved_products_count: int = ...,
        ) -> None: ...

    class ShoppingMerchantCenterAccountSuspensionRecommendation(proto.Message):
        merchant: Recommendation.MerchantInfo
        feed_label: str
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            merchant: Recommendation.MerchantInfo = ...,
            feed_label: str = ...,
        ) -> None: ...

    class ShoppingMigrateRegularShoppingCampaignOffersToPerformanceMaxRecommendation(
        proto.Message
    ):
        merchant: Recommendation.MerchantInfo
        feed_label: str
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            merchant: Recommendation.MerchantInfo = ...,
            feed_label: str = ...,
        ) -> None: ...

    class ShoppingOfferAttributeRecommendation(proto.Message):
        merchant: Recommendation.MerchantInfo
        feed_label: str
        offers_count: int
        demoted_offers_count: int
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            merchant: Recommendation.MerchantInfo = ...,
            feed_label: str = ...,
            offers_count: int = ...,
            demoted_offers_count: int = ...,
        ) -> None: ...

    class ShoppingTargetAllOffersRecommendation(proto.Message):
        merchant: Recommendation.MerchantInfo
        untargeted_offers_count: int
        feed_label: str
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            merchant: Recommendation.MerchantInfo = ...,
            untargeted_offers_count: int = ...,
            feed_label: str = ...,
        ) -> None: ...

    class SitelinkAssetRecommendation(proto.Message):
        recommended_campaign_sitelink_assets: MutableSequence[Asset]
        recommended_customer_sitelink_assets: MutableSequence[Asset]
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            recommended_campaign_sitelink_assets: MutableSequence[Asset] = ...,
            recommended_customer_sitelink_assets: MutableSequence[Asset] = ...,
        ) -> None: ...

    class TargetAdjustmentInfo(proto.Message):
        shared_set: str
        recommended_target_multiplier: float
        current_average_target_micros: int
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            shared_set: str = ...,
            recommended_target_multiplier: float = ...,
            current_average_target_micros: int = ...,
        ) -> None: ...

    class TargetCpaOptInRecommendation(proto.Message):
        class TargetCpaOptInRecommendationOption(proto.Message):
            goal: TargetCpaOptInRecommendationGoalEnum.TargetCpaOptInRecommendationGoal
            target_cpa_micros: int
            required_campaign_budget_amount_micros: int
            impact: Recommendation.RecommendationImpact
            def __init__(
                self: _M,
                mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
                *,
                ignore_unknown_fields: bool = ...,
                goal: TargetCpaOptInRecommendationGoalEnum.TargetCpaOptInRecommendationGoal = ...,
                target_cpa_micros: int = ...,
                required_campaign_budget_amount_micros: int = ...,
                impact: Recommendation.RecommendationImpact = ...,
            ) -> None: ...
        options: MutableSequence[
            Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption
        ]
        recommended_target_cpa_micros: int
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            options: MutableSequence[
                Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption
            ] = ...,
            recommended_target_cpa_micros: int = ...,
        ) -> None: ...

    class TargetRoasOptInRecommendation(proto.Message):
        recommended_target_roas: float
        required_campaign_budget_amount_micros: int
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            recommended_target_roas: float = ...,
            required_campaign_budget_amount_micros: int = ...,
        ) -> None: ...

    class TextAdRecommendation(proto.Message):
        ad: Ad
        creation_date: str
        auto_apply_date: str
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            ad: Ad = ...,
            creation_date: str = ...,
            auto_apply_date: str = ...,
        ) -> None: ...

    class UpgradeLocalCampaignToPerformanceMaxRecommendation(proto.Message):
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
        ) -> None: ...
        ...

    class UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation(proto.Message):
        merchant_id: int
        sales_country_code: str
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            merchant_id: int = ...,
            sales_country_code: str = ...,
        ) -> None: ...

    class UseBroadMatchKeywordRecommendation(proto.Message):
        keyword: MutableSequence[KeywordInfo]
        suggested_keywords_count: int
        campaign_keywords_count: int
        campaign_uses_shared_budget: bool
        required_campaign_budget_amount_micros: int
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            keyword: MutableSequence[KeywordInfo] = ...,
            suggested_keywords_count: int = ...,
            campaign_keywords_count: int = ...,
            campaign_uses_shared_budget: bool = ...,
            required_campaign_budget_amount_micros: int = ...,
        ) -> None: ...
    resource_name: str
    type_: RecommendationTypeEnum.RecommendationType
    impact: Recommendation.RecommendationImpact
    campaign_budget: str
    campaign: str
    ad_group: str
    dismissed: bool
    campaigns: MutableSequence[str]
    campaign_budget_recommendation: Recommendation.CampaignBudgetRecommendation
    forecasting_campaign_budget_recommendation: Recommendation.CampaignBudgetRecommendation
    keyword_recommendation: Recommendation.KeywordRecommendation
    text_ad_recommendation: Recommendation.TextAdRecommendation
    target_cpa_opt_in_recommendation: Recommendation.TargetCpaOptInRecommendation
    maximize_conversions_opt_in_recommendation: Recommendation.MaximizeConversionsOptInRecommendation
    enhanced_cpc_opt_in_recommendation: Recommendation.EnhancedCpcOptInRecommendation
    search_partners_opt_in_recommendation: Recommendation.SearchPartnersOptInRecommendation
    maximize_clicks_opt_in_recommendation: Recommendation.MaximizeClicksOptInRecommendation
    optimize_ad_rotation_recommendation: Recommendation.OptimizeAdRotationRecommendation
    keyword_match_type_recommendation: Recommendation.KeywordMatchTypeRecommendation
    move_unused_budget_recommendation: Recommendation.MoveUnusedBudgetRecommendation
    target_roas_opt_in_recommendation: Recommendation.TargetRoasOptInRecommendation
    responsive_search_ad_recommendation: Recommendation.ResponsiveSearchAdRecommendation
    marginal_roi_campaign_budget_recommendation: Recommendation.CampaignBudgetRecommendation
    use_broad_match_keyword_recommendation: Recommendation.UseBroadMatchKeywordRecommendation
    responsive_search_ad_asset_recommendation: Recommendation.ResponsiveSearchAdAssetRecommendation
    upgrade_smart_shopping_campaign_to_performance_max_recommendation: Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation
    responsive_search_ad_improve_ad_strength_recommendation: Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation
    display_expansion_opt_in_recommendation: Recommendation.DisplayExpansionOptInRecommendation
    upgrade_local_campaign_to_performance_max_recommendation: Recommendation.UpgradeLocalCampaignToPerformanceMaxRecommendation
    raise_target_cpa_bid_too_low_recommendation: Recommendation.RaiseTargetCpaBidTooLowRecommendation
    forecasting_set_target_roas_recommendation: Recommendation.ForecastingSetTargetRoasRecommendation
    callout_asset_recommendation: Recommendation.CalloutAssetRecommendation
    sitelink_asset_recommendation: Recommendation.SitelinkAssetRecommendation
    call_asset_recommendation: Recommendation.CallAssetRecommendation
    shopping_add_age_group_recommendation: Recommendation.ShoppingOfferAttributeRecommendation
    shopping_add_color_recommendation: Recommendation.ShoppingOfferAttributeRecommendation
    shopping_add_gender_recommendation: Recommendation.ShoppingOfferAttributeRecommendation
    shopping_add_gtin_recommendation: Recommendation.ShoppingOfferAttributeRecommendation
    shopping_add_more_identifiers_recommendation: Recommendation.ShoppingOfferAttributeRecommendation
    shopping_add_size_recommendation: Recommendation.ShoppingOfferAttributeRecommendation
    shopping_add_products_to_campaign_recommendation: Recommendation.ShoppingAddProductsToCampaignRecommendation
    shopping_fix_disapproved_products_recommendation: Recommendation.ShoppingFixDisapprovedProductsRecommendation
    shopping_target_all_offers_recommendation: Recommendation.ShoppingTargetAllOffersRecommendation
    shopping_fix_suspended_merchant_center_account_recommendation: Recommendation.ShoppingMerchantCenterAccountSuspensionRecommendation
    shopping_fix_merchant_center_account_suspension_warning_recommendation: Recommendation.ShoppingMerchantCenterAccountSuspensionRecommendation
    shopping_migrate_regular_shopping_campaign_offers_to_performance_max_recommendation: Recommendation.ShoppingMigrateRegularShoppingCampaignOffersToPerformanceMaxRecommendation
    dynamic_image_extension_opt_in_recommendation: Recommendation.DynamicImageExtensionOptInRecommendation
    raise_target_cpa_recommendation: Recommendation.RaiseTargetCpaRecommendation
    lower_target_roas_recommendation: Recommendation.LowerTargetRoasRecommendation
    performance_max_opt_in_recommendation: Recommendation.PerformanceMaxOptInRecommendation
    improve_performance_max_ad_strength_recommendation: Recommendation.ImprovePerformanceMaxAdStrengthRecommendation
    migrate_dynamic_search_ads_campaign_to_performance_max_recommendation: Recommendation.MigrateDynamicSearchAdsCampaignToPerformanceMaxRecommendation
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        resource_name: str = ...,
        type_: RecommendationTypeEnum.RecommendationType = ...,
        impact: Recommendation.RecommendationImpact = ...,
        campaign_budget: str = ...,
        campaign: str = ...,
        ad_group: str = ...,
        dismissed: bool = ...,
        campaigns: MutableSequence[str] = ...,
        campaign_budget_recommendation: Recommendation.CampaignBudgetRecommendation = ...,
        forecasting_campaign_budget_recommendation: Recommendation.CampaignBudgetRecommendation = ...,
        keyword_recommendation: Recommendation.KeywordRecommendation = ...,
        text_ad_recommendation: Recommendation.TextAdRecommendation = ...,
        target_cpa_opt_in_recommendation: Recommendation.TargetCpaOptInRecommendation = ...,
        maximize_conversions_opt_in_recommendation: Recommendation.MaximizeConversionsOptInRecommendation = ...,
        enhanced_cpc_opt_in_recommendation: Recommendation.EnhancedCpcOptInRecommendation = ...,
        search_partners_opt_in_recommendation: Recommendation.SearchPartnersOptInRecommendation = ...,
        maximize_clicks_opt_in_recommendation: Recommendation.MaximizeClicksOptInRecommendation = ...,
        optimize_ad_rotation_recommendation: Recommendation.OptimizeAdRotationRecommendation = ...,
        keyword_match_type_recommendation: Recommendation.KeywordMatchTypeRecommendation = ...,
        move_unused_budget_recommendation: Recommendation.MoveUnusedBudgetRecommendation = ...,
        target_roas_opt_in_recommendation: Recommendation.TargetRoasOptInRecommendation = ...,
        responsive_search_ad_recommendation: Recommendation.ResponsiveSearchAdRecommendation = ...,
        marginal_roi_campaign_budget_recommendation: Recommendation.CampaignBudgetRecommendation = ...,
        use_broad_match_keyword_recommendation: Recommendation.UseBroadMatchKeywordRecommendation = ...,
        responsive_search_ad_asset_recommendation: Recommendation.ResponsiveSearchAdAssetRecommendation = ...,
        upgrade_smart_shopping_campaign_to_performance_max_recommendation: Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation = ...,
        responsive_search_ad_improve_ad_strength_recommendation: Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation = ...,
        display_expansion_opt_in_recommendation: Recommendation.DisplayExpansionOptInRecommendation = ...,
        upgrade_local_campaign_to_performance_max_recommendation: Recommendation.UpgradeLocalCampaignToPerformanceMaxRecommendation = ...,
        raise_target_cpa_bid_too_low_recommendation: Recommendation.RaiseTargetCpaBidTooLowRecommendation = ...,
        forecasting_set_target_roas_recommendation: Recommendation.ForecastingSetTargetRoasRecommendation = ...,
        callout_asset_recommendation: Recommendation.CalloutAssetRecommendation = ...,
        sitelink_asset_recommendation: Recommendation.SitelinkAssetRecommendation = ...,
        call_asset_recommendation: Recommendation.CallAssetRecommendation = ...,
        shopping_add_age_group_recommendation: Recommendation.ShoppingOfferAttributeRecommendation = ...,
        shopping_add_color_recommendation: Recommendation.ShoppingOfferAttributeRecommendation = ...,
        shopping_add_gender_recommendation: Recommendation.ShoppingOfferAttributeRecommendation = ...,
        shopping_add_gtin_recommendation: Recommendation.ShoppingOfferAttributeRecommendation = ...,
        shopping_add_more_identifiers_recommendation: Recommendation.ShoppingOfferAttributeRecommendation = ...,
        shopping_add_size_recommendation: Recommendation.ShoppingOfferAttributeRecommendation = ...,
        shopping_add_products_to_campaign_recommendation: Recommendation.ShoppingAddProductsToCampaignRecommendation = ...,
        shopping_fix_disapproved_products_recommendation: Recommendation.ShoppingFixDisapprovedProductsRecommendation = ...,
        shopping_target_all_offers_recommendation: Recommendation.ShoppingTargetAllOffersRecommendation = ...,
        shopping_fix_suspended_merchant_center_account_recommendation: Recommendation.ShoppingMerchantCenterAccountSuspensionRecommendation = ...,
        shopping_fix_merchant_center_account_suspension_warning_recommendation: Recommendation.ShoppingMerchantCenterAccountSuspensionRecommendation = ...,
        shopping_migrate_regular_shopping_campaign_offers_to_performance_max_recommendation: Recommendation.ShoppingMigrateRegularShoppingCampaignOffersToPerformanceMaxRecommendation = ...,
        dynamic_image_extension_opt_in_recommendation: Recommendation.DynamicImageExtensionOptInRecommendation = ...,
        raise_target_cpa_recommendation: Recommendation.RaiseTargetCpaRecommendation = ...,
        lower_target_roas_recommendation: Recommendation.LowerTargetRoasRecommendation = ...,
        performance_max_opt_in_recommendation: Recommendation.PerformanceMaxOptInRecommendation = ...,
        improve_performance_max_ad_strength_recommendation: Recommendation.ImprovePerformanceMaxAdStrengthRecommendation = ...,
        migrate_dynamic_search_ads_campaign_to_performance_max_recommendation: Recommendation.MigrateDynamicSearchAdsCampaignToPerformanceMaxRecommendation = ...,
    ) -> None: ...



from collections.abc import Mapping
from typing import Any, TypeVar

import google.protobuf.message
import proto

_M = TypeVar("_M")

class KeywordPlanAdGroupKeywordErrorEnum(proto.Message):
    class KeywordPlanAdGroupKeywordError(proto.Enum):
        UNSPECIFIED = 0
        UNKNOWN = 1
        INVALID_KEYWORD_MATCH_TYPE = 2
        DUPLICATE_KEYWORD = 3
        KEYWORD_TEXT_TOO_LONG = 4
        KEYWORD_HAS_INVALID_CHARS = 5
        KEYWORD_HAS_TOO_MANY_WORDS = 6
        INVALID_KEYWORD_TEXT = 7
        NEGATIVE_KEYWORD_HAS_CPC_BID = 8
        NEW_BMM_KEYWORDS_NOT_ALLOWED = 9
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
    ) -> None: ...



from collections.abc import Mapping
from typing import Any, TypeVar

import google.protobuf.message
import proto

_M = TypeVar("_M")

class AssetLinkErrorEnum(proto.Message):
    class AssetLinkError(proto.Enum):
        UNSPECIFIED = 0
        UNKNOWN = 1
        PINNING_UNSUPPORTED = 2
        UNSUPPORTED_FIELD_TYPE = 3
        FIELD_TYPE_INCOMPATIBLE_WITH_ASSET_TYPE = 4
        FIELD_TYPE_INCOMPATIBLE_WITH_CAMPAIGN_TYPE = 5
        INCOMPATIBLE_ADVERTISING_CHANNEL_TYPE = 6
        IMAGE_NOT_WITHIN_SPECIFIED_DIMENSION_RANGE = 7
        INVALID_PINNED_FIELD = 8
        MEDIA_BUNDLE_ASSET_FILE_SIZE_TOO_LARGE = 9
        NOT_ENOUGH_AVAILABLE_ASSET_LINKS_FOR_VALID_COMBINATION = 10
        NOT_ENOUGH_AVAILABLE_ASSET_LINKS_WITH_FALLBACK = 11
        NOT_ENOUGH_AVAILABLE_ASSET_LINKS_WITH_FALLBACK_FOR_VALID_COMBINATION = 12
        YOUTUBE_VIDEO_REMOVED = 13
        YOUTUBE_VIDEO_TOO_LONG = 14
        YOUTUBE_VIDEO_TOO_SHORT = 15
        EXCLUDED_PARENT_FIELD_TYPE = 16
        INVALID_STATUS = 17
        YOUTUBE_VIDEO_DURATION_NOT_DEFINED = 18
        CANNOT_CREATE_AUTOMATICALLY_CREATED_LINKS = 19
        CANNOT_LINK_TO_AUTOMATICALLY_CREATED_ASSET = 20
        CANNOT_MODIFY_ASSET_LINK_SOURCE = 21
        CANNOT_LINK_LOCATION_LEAD_FORM_WITHOUT_LOCATION_ASSET = 22
        CUSTOMER_NOT_VERIFIED = 23
        UNSUPPORTED_CALL_TO_ACTION = 24
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
    ) -> None: ...



from collections.abc import Mapping
from typing import Any, TypeVar

import google.protobuf.message
import proto

from google.ads.googleads.v13.common.types.criteria import (
    AdScheduleInfo,
    AgeRangeInfo,
    CarrierInfo,
    CombinedAudienceInfo,
    ContentLabelInfo,
    CustomAffinityInfo,
    CustomAudienceInfo,
    DeviceInfo,
    GenderInfo,
    IncomeRangeInfo,
    IpBlockInfo,
    KeywordInfo,
    KeywordThemeInfo,
    LanguageInfo,
    ListingScopeInfo,
    LocalServiceIdInfo,
    LocationGroupInfo,
    LocationInfo,
    MobileAppCategoryInfo,
    MobileApplicationInfo,
    MobileDeviceInfo,
    OperatingSystemVersionInfo,
    ParentalStatusInfo,
    PlacementInfo,
    ProximityInfo,
    TopicInfo,
    UserInterestInfo,
    UserListInfo,
    WebpageInfo,
    YouTubeChannelInfo,
    YouTubeVideoInfo,
)
from google.ads.googleads.v13.enums.types.campaign_criterion_status import (
    CampaignCriterionStatusEnum,
)
from google.ads.googleads.v13.enums.types.criterion_type import CriterionTypeEnum

_M = TypeVar("_M")

class CampaignCriterion(proto.Message):
    resource_name: str
    campaign: str
    criterion_id: int
    display_name: str
    bid_modifier: float
    negative: bool
    type_: CriterionTypeEnum.CriterionType
    status: CampaignCriterionStatusEnum.CampaignCriterionStatus
    keyword: KeywordInfo
    placement: PlacementInfo
    mobile_app_category: MobileAppCategoryInfo
    mobile_application: MobileApplicationInfo
    location: LocationInfo
    device: DeviceInfo
    ad_schedule: AdScheduleInfo
    age_range: AgeRangeInfo
    gender: GenderInfo
    income_range: IncomeRangeInfo
    parental_status: ParentalStatusInfo
    user_list: UserListInfo
    youtube_video: YouTubeVideoInfo
    youtube_channel: YouTubeChannelInfo
    proximity: ProximityInfo
    topic: TopicInfo
    listing_scope: ListingScopeInfo
    language: LanguageInfo
    ip_block: IpBlockInfo
    content_label: ContentLabelInfo
    carrier: CarrierInfo
    user_interest: UserInterestInfo
    webpage: WebpageInfo
    operating_system_version: OperatingSystemVersionInfo
    mobile_device: MobileDeviceInfo
    location_group: LocationGroupInfo
    custom_affinity: CustomAffinityInfo
    custom_audience: CustomAudienceInfo
    combined_audience: CombinedAudienceInfo
    keyword_theme: KeywordThemeInfo
    local_service_id: LocalServiceIdInfo
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        resource_name: str = ...,
        campaign: str = ...,
        criterion_id: int = ...,
        display_name: str = ...,
        bid_modifier: float = ...,
        negative: bool = ...,
        type_: CriterionTypeEnum.CriterionType = ...,
        status: CampaignCriterionStatusEnum.CampaignCriterionStatus = ...,
        keyword: KeywordInfo = ...,
        placement: PlacementInfo = ...,
        mobile_app_category: MobileAppCategoryInfo = ...,
        mobile_application: MobileApplicationInfo = ...,
        location: LocationInfo = ...,
        device: DeviceInfo = ...,
        ad_schedule: AdScheduleInfo = ...,
        age_range: AgeRangeInfo = ...,
        gender: GenderInfo = ...,
        income_range: IncomeRangeInfo = ...,
        parental_status: ParentalStatusInfo = ...,
        user_list: UserListInfo = ...,
        youtube_video: YouTubeVideoInfo = ...,
        youtube_channel: YouTubeChannelInfo = ...,
        proximity: ProximityInfo = ...,
        topic: TopicInfo = ...,
        listing_scope: ListingScopeInfo = ...,
        language: LanguageInfo = ...,
        ip_block: IpBlockInfo = ...,
        content_label: ContentLabelInfo = ...,
        carrier: CarrierInfo = ...,
        user_interest: UserInterestInfo = ...,
        webpage: WebpageInfo = ...,
        operating_system_version: OperatingSystemVersionInfo = ...,
        mobile_device: MobileDeviceInfo = ...,
        location_group: LocationGroupInfo = ...,
        custom_affinity: CustomAffinityInfo = ...,
        custom_audience: CustomAudienceInfo = ...,
        combined_audience: CombinedAudienceInfo = ...,
        keyword_theme: KeywordThemeInfo = ...,
        local_service_id: LocalServiceIdInfo = ...
    ) -> None: ...



from collections.abc import Mapping, MutableSequence
from typing import Any, TypeVar

import google.protobuf.message
import proto

from google.ads.googleads.v13.resources.types.geo_target_constant import (
    GeoTargetConstant,
)

_M = TypeVar("_M")

class GeoTargetConstantSuggestion(proto.Message):
    locale: str
    reach: int
    search_term: str
    geo_target_constant: GeoTargetConstant
    geo_target_constant_parents: MutableSequence[GeoTargetConstant]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        locale: str = ...,
        reach: int = ...,
        search_term: str = ...,
        geo_target_constant: GeoTargetConstant = ...,
        geo_target_constant_parents: MutableSequence[GeoTargetConstant] = ...
    ) -> None: ...

class SuggestGeoTargetConstantsRequest(proto.Message):
    class GeoTargets(proto.Message):
        geo_target_constants: MutableSequence[str]
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            geo_target_constants: MutableSequence[str] = ...
        ) -> None: ...

    class LocationNames(proto.Message):
        names: MutableSequence[str]
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            names: MutableSequence[str] = ...
        ) -> None: ...
    locale: str
    country_code: str
    location_names: SuggestGeoTargetConstantsRequest.LocationNames
    geo_targets: SuggestGeoTargetConstantsRequest.GeoTargets
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        locale: str = ...,
        country_code: str = ...,
        location_names: SuggestGeoTargetConstantsRequest.LocationNames = ...,
        geo_targets: SuggestGeoTargetConstantsRequest.GeoTargets = ...
    ) -> None: ...

class SuggestGeoTargetConstantsResponse(proto.Message):
    geo_target_constant_suggestions: MutableSequence[GeoTargetConstantSuggestion]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        geo_target_constant_suggestions: MutableSequence[
            GeoTargetConstantSuggestion
        ] = ...
    ) -> None: ...



from collections.abc import Mapping
from typing import Any, TypeVar

import google.protobuf.message
import proto

_M = TypeVar("_M")

class CustomConversionGoalErrorEnum(proto.Message):
    class CustomConversionGoalError(proto.Enum):
        UNSPECIFIED = 0
        UNKNOWN = 1
        INVALID_CONVERSION_ACTION = 2
        CONVERSION_ACTION_NOT_ENABLED = 3
        CANNOT_REMOVE_LINKED_CUSTOM_CONVERSION_GOAL = 4
        CUSTOM_GOAL_DUPLICATE_NAME = 5
        DUPLICATE_CONVERSION_ACTION_LIST = 6
        NON_BIDDABLE_CONVERSION_ACTION_NOT_ELIGIBLE_FOR_CUSTOM_GOAL = 7
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
    ) -> None: ...



from collections.abc import Mapping
from typing import Any, TypeVar

import google.protobuf.message
import proto

_M = TypeVar("_M")

class AppCampaignBiddingStrategyGoalTypeEnum(proto.Message):
    class AppCampaignBiddingStrategyGoalType(proto.Enum):
        UNSPECIFIED = 0
        UNKNOWN = 1
        OPTIMIZE_INSTALLS_TARGET_INSTALL_COST = 2
        OPTIMIZE_IN_APP_CONVERSIONS_TARGET_INSTALL_COST = 3
        OPTIMIZE_IN_APP_CONVERSIONS_TARGET_CONVERSION_COST = 4
        OPTIMIZE_RETURN_ON_ADVERTISING_SPEND = 5
        OPTIMIZE_PRE_REGISTRATION_CONVERSION_VOLUME = 6
        OPTIMIZE_INSTALLS_WITHOUT_TARGET_INSTALL_COST = 7
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
    ) -> None: ...



from collections.abc import Mapping, MutableSequence
from typing import Any, TypeVar

import google.protobuf.message
import proto
from google.rpc.status_pb2 import Status

from google.ads.googleads.v13.common.types.offline_user_data import UserIdentifier
from google.ads.googleads.v13.enums.types.conversion_environment_enum import (
    ConversionEnvironmentEnum,
)

_M = TypeVar("_M")

class CallConversion(proto.Message):
    caller_id: str
    call_start_date_time: str
    conversion_action: str
    conversion_date_time: str
    conversion_value: float
    currency_code: str
    custom_variables: MutableSequence[CustomVariable]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        caller_id: str = ...,
        call_start_date_time: str = ...,
        conversion_action: str = ...,
        conversion_date_time: str = ...,
        conversion_value: float = ...,
        currency_code: str = ...,
        custom_variables: MutableSequence[CustomVariable] = ...
    ) -> None: ...

class CallConversionResult(proto.Message):
    caller_id: str
    call_start_date_time: str
    conversion_action: str
    conversion_date_time: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        caller_id: str = ...,
        call_start_date_time: str = ...,
        conversion_action: str = ...,
        conversion_date_time: str = ...
    ) -> None: ...

class CartData(proto.Message):
    class Item(proto.Message):
        product_id: str
        quantity: int
        unit_price: float
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
            *,
            ignore_unknown_fields: bool = ...,
            product_id: str = ...,
            quantity: int = ...,
            unit_price: float = ...
        ) -> None: ...
    merchant_id: int
    feed_country_code: str
    feed_language_code: str
    local_transaction_cost: float
    items: MutableSequence[CartData.Item]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        merchant_id: int = ...,
        feed_country_code: str = ...,
        feed_language_code: str = ...,
        local_transaction_cost: float = ...,
        items: MutableSequence[CartData.Item] = ...
    ) -> None: ...

class ClickConversion(proto.Message):
    gclid: str
    gbraid: str
    wbraid: str
    conversion_action: str
    conversion_date_time: str
    conversion_value: float
    currency_code: str
    order_id: str
    external_attribution_data: ExternalAttributionData
    custom_variables: MutableSequence[CustomVariable]
    cart_data: CartData
    user_identifiers: MutableSequence[UserIdentifier]
    conversion_environment: ConversionEnvironmentEnum.ConversionEnvironment
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        gclid: str = ...,
        gbraid: str = ...,
        wbraid: str = ...,
        conversion_action: str = ...,
        conversion_date_time: str = ...,
        conversion_value: float = ...,
        currency_code: str = ...,
        order_id: str = ...,
        external_attribution_data: ExternalAttributionData = ...,
        custom_variables: MutableSequence[CustomVariable] = ...,
        cart_data: CartData = ...,
        user_identifiers: MutableSequence[UserIdentifier] = ...,
        conversion_environment: ConversionEnvironmentEnum.ConversionEnvironment = ...
    ) -> None: ...

class ClickConversionResult(proto.Message):
    gclid: str
    gbraid: str
    wbraid: str
    conversion_action: str
    conversion_date_time: str
    user_identifiers: MutableSequence[UserIdentifier]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        gclid: str = ...,
        gbraid: str = ...,
        wbraid: str = ...,
        conversion_action: str = ...,
        conversion_date_time: str = ...,
        user_identifiers: MutableSequence[UserIdentifier] = ...
    ) -> None: ...

class CustomVariable(proto.Message):
    conversion_custom_variable: str
    value: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        conversion_custom_variable: str = ...,
        value: str = ...
    ) -> None: ...

class ExternalAttributionData(proto.Message):
    external_attribution_credit: float
    external_attribution_model: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        external_attribution_credit: float = ...,
        external_attribution_model: str = ...
    ) -> None: ...

class UploadCallConversionsRequest(proto.Message):
    customer_id: str
    conversions: MutableSequence[CallConversion]
    partial_failure: bool
    validate_only: bool
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        customer_id: str = ...,
        conversions: MutableSequence[CallConversion] = ...,
        partial_failure: bool = ...,
        validate_only: bool = ...
    ) -> None: ...

class UploadCallConversionsResponse(proto.Message):
    partial_failure_error: Status
    results: MutableSequence[CallConversionResult]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        partial_failure_error: Status = ...,
        results: MutableSequence[CallConversionResult] = ...
    ) -> None: ...

class UploadClickConversionsRequest(proto.Message):
    customer_id: str
    conversions: MutableSequence[ClickConversion]
    partial_failure: bool
    validate_only: bool
    debug_enabled: bool
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        customer_id: str = ...,
        conversions: MutableSequence[ClickConversion] = ...,
        partial_failure: bool = ...,
        validate_only: bool = ...,
        debug_enabled: bool = ...
    ) -> None: ...

class UploadClickConversionsResponse(proto.Message):
    partial_failure_error: Status
    results: MutableSequence[ClickConversionResult]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        partial_failure_error: Status = ...,
        results: MutableSequence[ClickConversionResult] = ...
    ) -> None: ...



from collections.abc import Mapping, MutableSequence
from typing import Any, TypeVar

import google.protobuf.message
import proto

from google.ads.googleads.v14.enums.types.user_identifier_source import (
    UserIdentifierSourceEnum,
)

_M = TypeVar("_M")

class CustomerMatchUserListMetadata(proto.Message):
    user_list: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        user_list: str = ...
    ) -> None: ...

class EventAttribute(proto.Message):
    event: str
    event_date_time: str
    item_attribute: MutableSequence[EventItemAttribute]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        event: str = ...,
        event_date_time: str = ...,
        item_attribute: MutableSequence[EventItemAttribute] = ...
    ) -> None: ...

class EventItemAttribute(proto.Message):
    item_id: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        item_id: str = ...
    ) -> None: ...

class ItemAttribute(proto.Message):
    item_id: str
    merchant_id: int
    country_code: str
    language_code: str
    quantity: int
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        item_id: str = ...,
        merchant_id: int = ...,
        country_code: str = ...,
        language_code: str = ...,
        quantity: int = ...
    ) -> None: ...

class OfflineUserAddressInfo(proto.Message):
    hashed_first_name: str
    hashed_last_name: str
    city: str
    state: str
    country_code: str
    postal_code: str
    hashed_street_address: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        hashed_first_name: str = ...,
        hashed_last_name: str = ...,
        city: str = ...,
        state: str = ...,
        country_code: str = ...,
        postal_code: str = ...,
        hashed_street_address: str = ...
    ) -> None: ...

class ShoppingLoyalty(proto.Message):
    loyalty_tier: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        loyalty_tier: str = ...
    ) -> None: ...

class StoreAttribute(proto.Message):
    store_code: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        store_code: str = ...
    ) -> None: ...

class StoreSalesMetadata(proto.Message):
    loyalty_fraction: float
    transaction_upload_fraction: float
    custom_key: str
    third_party_metadata: StoreSalesThirdPartyMetadata
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        loyalty_fraction: float = ...,
        transaction_upload_fraction: float = ...,
        custom_key: str = ...,
        third_party_metadata: StoreSalesThirdPartyMetadata = ...
    ) -> None: ...

class StoreSalesThirdPartyMetadata(proto.Message):
    advertiser_upload_date_time: str
    valid_transaction_fraction: float
    partner_match_fraction: float
    partner_upload_fraction: float
    bridge_map_version_id: str
    partner_id: int
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        advertiser_upload_date_time: str = ...,
        valid_transaction_fraction: float = ...,
        partner_match_fraction: float = ...,
        partner_upload_fraction: float = ...,
        bridge_map_version_id: str = ...,
        partner_id: int = ...
    ) -> None: ...

class TransactionAttribute(proto.Message):
    transaction_date_time: str
    transaction_amount_micros: float
    currency_code: str
    conversion_action: str
    order_id: str
    store_attribute: StoreAttribute
    custom_value: str
    item_attribute: ItemAttribute
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        transaction_date_time: str = ...,
        transaction_amount_micros: float = ...,
        currency_code: str = ...,
        conversion_action: str = ...,
        order_id: str = ...,
        store_attribute: StoreAttribute = ...,
        custom_value: str = ...,
        item_attribute: ItemAttribute = ...
    ) -> None: ...

class UserAttribute(proto.Message):
    lifetime_value_micros: int
    lifetime_value_bucket: int
    last_purchase_date_time: str
    average_purchase_count: int
    average_purchase_value_micros: int
    acquisition_date_time: str
    shopping_loyalty: ShoppingLoyalty
    lifecycle_stage: str
    first_purchase_date_time: str
    event_attribute: MutableSequence[EventAttribute]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        lifetime_value_micros: int = ...,
        lifetime_value_bucket: int = ...,
        last_purchase_date_time: str = ...,
        average_purchase_count: int = ...,
        average_purchase_value_micros: int = ...,
        acquisition_date_time: str = ...,
        shopping_loyalty: ShoppingLoyalty = ...,
        lifecycle_stage: str = ...,
        first_purchase_date_time: str = ...,
        event_attribute: MutableSequence[EventAttribute] = ...
    ) -> None: ...

class UserData(proto.Message):
    user_identifiers: MutableSequence[UserIdentifier]
    transaction_attribute: TransactionAttribute
    user_attribute: UserAttribute
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        user_identifiers: MutableSequence[UserIdentifier] = ...,
        transaction_attribute: TransactionAttribute = ...,
        user_attribute: UserAttribute = ...
    ) -> None: ...

class UserIdentifier(proto.Message):
    user_identifier_source: UserIdentifierSourceEnum.UserIdentifierSource
    hashed_email: str
    hashed_phone_number: str
    mobile_id: str
    third_party_user_id: str
    address_info: OfflineUserAddressInfo
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        user_identifier_source: UserIdentifierSourceEnum.UserIdentifierSource = ...,
        hashed_email: str = ...,
        hashed_phone_number: str = ...,
        mobile_id: str = ...,
        third_party_user_id: str = ...,
        address_info: OfflineUserAddressInfo = ...
    ) -> None: ...



from collections.abc import Mapping, MutableSequence
from typing import Any, TypeVar

import google.protobuf.message
import proto

from google.ads.googleads.v14.enums.types.custom_audience_member_type import (
    CustomAudienceMemberTypeEnum,
)
from google.ads.googleads.v14.enums.types.custom_audience_status import (
    CustomAudienceStatusEnum,
)
from google.ads.googleads.v14.enums.types.custom_audience_type import (
    CustomAudienceTypeEnum,
)

_M = TypeVar("_M")

class CustomAudience(proto.Message):
    resource_name: str
    id: int
    status: CustomAudienceStatusEnum.CustomAudienceStatus
    name: str
    type_: CustomAudienceTypeEnum.CustomAudienceType
    description: str
    members: MutableSequence[CustomAudienceMember]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        resource_name: str = ...,
        id: int = ...,
        status: CustomAudienceStatusEnum.CustomAudienceStatus = ...,
        name: str = ...,
        type_: CustomAudienceTypeEnum.CustomAudienceType = ...,
        description: str = ...,
        members: MutableSequence[CustomAudienceMember] = ...
    ) -> None: ...

class CustomAudienceMember(proto.Message):
    member_type: CustomAudienceMemberTypeEnum.CustomAudienceMemberType
    keyword: str
    url: str
    place_category: int
    app: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = ...,
        *,
        ignore_unknown_fields: bool = ...,
        member_type: CustomAudienceMemberTypeEnum.CustomAudienceMemberType = ...,
        keyword: str = ...,
        url: str = ...,
        place_category: int = ...,
        app: str = ...
    ) -> None: ...
